@@ -50,22 +50,23 @@ namespace llvm {
 // Pass initialization functions need to be declared before inclusion of
 // PassSupport.h.
 class PassRegistry;
-void initializeLLVMToSPIRVPass(PassRegistry &);
-void initializeOCL20ToSPIRVPass(PassRegistry &);
-void initializeOCL21ToSPIRVPass(PassRegistry &);
-void initializeOCLTypeToSPIRVPass(PassRegistry &);
-void initializeSPIRVLowerBoolPass(PassRegistry &);
-void initializeSPIRVLowerConstExprPass(PassRegistry &);
-void initializeSPIRVLowerSPIRBlocksPass(PassRegistry &);
-void initializeSPIRVLowerOCLBlocksPass(PassRegistry &);
-void initializeSPIRVLowerMemmovePass(PassRegistry &);
-void initializeSPIRVLowerSaddWithOverflowPass(PassRegistry &);
-void initializeSPIRVRegularizeLLVMPass(PassRegistry &);
-void initializeSPIRVToOCL12Pass(PassRegistry &);
-void initializeSPIRVToOCL20Pass(PassRegistry &);
-void initializePreprocessMetadataPass(PassRegistry &);
+void initializeLLVMToSPIRVLegacyPass(PassRegistry &);
+void initializeOCLToSPIRVLegacyPass(PassRegistry &);
+void initializeOCLTypeToSPIRVLegacyPass(PassRegistry &);
+void initializeSPIRVLowerBoolLegacyPass(PassRegistry &);
+void initializeSPIRVLowerConstExprLegacyPass(PassRegistry &);
+void initializeSPIRVLowerSPIRBlocksLegacyPass(PassRegistry &);
+void initializeSPIRVLowerOCLBlocksLegacyPass(PassRegistry &);
+void initializeSPIRVLowerMemmoveLegacyPass(PassRegistry &);
+void initializeSPIRVLowerSaddWithOverflowLegacyPass(PassRegistry &);
+void initializeSPIRVRegularizeLLVMLegacyPass(PassRegistry &);
+void initializeSPIRVToOCL12LegacyPass(PassRegistry &);
+void initializeSPIRVToOCL20LegacyPass(PassRegistry &);
+void initializePreprocessMetadataLegacyPass(PassRegistry &);
+void initializeSPIRVLowerBitCastToNonStandardTypeLegacyPass(PassRegistry &);
 
 class ModulePass;
+class FunctionPass;
 } // namespace llvm
 
 #include "llvm/IR/Module.h"
@@ -75,7 +76,7 @@ namespace SPIRV {
 class SPIRVModule;
 
 /// \brief Check if a string contains SPIR-V binary.
-bool isSpirvBinary(std::string &Img);
+bool isSpirvBinary(const std::string &Img);
 
 #ifdef _SPIRV_SUPPORT_TEXT_FMT
 /// \brief Convert SPIR-V between binary and internal textual formats.
@@ -160,40 +161,36 @@ void mangleOpenClBuiltin(const std::string &UnmangledName,
                          ArrayRef<Type *> ArgTypes, std::string &MangledName);
 
 /// Create a pass for translating LLVM to SPIR-V.
-ModulePass *createLLVMToSPIRV(SPIRV::SPIRVModule *);
+ModulePass *createLLVMToSPIRVLegacy(SPIRV::SPIRVModule *);
 
-/// Create a pass for translating OCL 2.0 builtin functions to SPIR-V builtin
+/// Create a pass for translating OCL C builtin functions to SPIR-V builtin
 /// functions.
-ModulePass *createOCL20ToSPIRV();
-
-/// Create a pass for translating OCL 2.1 builtin functions to SPIR-V builtin
-/// functions.
-ModulePass *createOCL21ToSPIRV();
+ModulePass *createOCLToSPIRVLegacy();
 
 /// Create a pass for adapting OCL types for SPIRV.
-ModulePass *createOCLTypeToSPIRV();
+ModulePass *createOCLTypeToSPIRVLegacy();
 
 /// Create a pass for lowering cast instructions of i1 type.
-ModulePass *createSPIRVLowerBool();
+ModulePass *createSPIRVLowerBoolLegacy();
 
 /// Create a pass for lowering constant expressions to instructions.
-ModulePass *createSPIRVLowerConstExpr();
+ModulePass *createSPIRVLowerConstExprLegacy();
 
 /// Create a pass for lowering SPIR 2.0 blocks to functions calls.
-ModulePass *createSPIRVLowerSPIRBlocks();
+ModulePass *createSPIRVLowerSPIRBlocksLegacy();
 
 /// Create a pass for removing function pointers related to OCL 2.0 blocks
-ModulePass *createSPIRVLowerOCLBlocks();
+ModulePass *createSPIRVLowerOCLBlocksLegacy();
 
 /// Create a pass for lowering llvm.memmove to llvm.memcpys with a temporary
 /// variable.
-ModulePass *createSPIRVLowerMemmove();
+ModulePass *createSPIRVLowerMemmoveLegacy();
 
 /// Create a pass for lowering llvm.sadd.with.overflow
-ModulePass *createSPIRVLowerSaddWithOverflow();
+ModulePass *createSPIRVLowerSaddWithOverflowLegacy();
 
 /// Create a pass for regularize LLVM module to be translated to SPIR-V.
-ModulePass *createSPIRVRegularizeLLVM();
+ModulePass *createSPIRVRegularizeLLVMLegacy();
 
 /// Create a pass for translating SPIR-V Instructions to desired
 /// representation in LLVM IR (OpenCL built-ins, SPIR-V Friendly IR, etc.)
@@ -201,15 +198,15 @@ ModulePass *createSPIRVBIsLoweringPass(Module &, SPIRV::BIsRepresentation);
 
 /// Create a pass for translating SPIR-V builtin functions to OCL 1.2 builtin
 /// functions.
-ModulePass *createSPIRVToOCL12();
+ModulePass *createSPIRVToOCL12Legacy();
 
 /// Create a pass for translating SPIR-V builtin functions to OCL 2.0 builtin
 /// functions.
-ModulePass *createSPIRVToOCL20();
+ModulePass *createSPIRVToOCL20Legacy();
 
 /// Create a pass for translating SPIR 1.2/2.0 metadata to SPIR-V friendly
 /// metadata.
-ModulePass *createPreprocessMetadata();
+ModulePass *createPreprocessMetadataLegacy();
 
 /// Create and return a pass that writes the module to the specified
 /// ostream.
@@ -220,6 +217,11 @@ ModulePass *createSPIRVWriterPass(std::ostream &Str);
 ModulePass *createSPIRVWriterPass(std::ostream &Str,
                                   const SPIRV::TranslatorOpts &Opts);
 
+/// Create a pass for removing bitcast instructions to non-standard SPIR-V
+/// types
+FunctionPass *createSPIRVLowerBitCastToNonStandardTypeLegacy(
+    const SPIRV::TranslatorOpts &Opts);
+
 } // namespace llvm
 
 #endif // SPIRV_H
