@@ -45,6 +45,7 @@
 #include "llvm/IR/Instructions.h"
 #include "llvm/Support/Path.h"
 
+#include <atomic>
 #include <functional>
 #include <tuple>
 #include <type_traits>
@@ -100,6 +101,13 @@ enum OCLMemOrderKind {
   OCLMO_seq_cst = std::memory_order::memory_order_seq_cst
 };
 
+enum IntelFPGAMemoryAccessesVal {
+  BurstCoalesce = 0x1,
+  CacheSizeFlag = 0x2,
+  DontStaticallyCoalesce = 0x4,
+  PrefetchFlag = 0x8
+};
+
 ///////////////////////////////////////////////////////////////////////////////
 //
 // Types
@@ -254,6 +262,8 @@ const static char IsNormal[] = "isnormal";
 const static char IsInf[] = "isinf";
 const static char Max[] = "max";
 const static char MemFence[] = "mem_fence";
+const static char ReadMemFence[] = "read_mem_fence";
+const static char WriteMemFence[] = "write_mem_fence";
 const static char Min[] = "min";
 const static char Mix[] = "mix";
 const static char NDRangePrefix[] = "ndrange_";
@@ -294,6 +304,27 @@ const static char SubgroupBlockWriteINTELPrefix[] =
     "intel_sub_group_block_write";
 const static char SubgroupImageMediaBlockINTELPrefix[] =
     "intel_sub_group_media_block";
+const static char LDEXP[] = "ldexp";
+#define _SPIRV_OP(x)                                                           \
+  const static char ConvertBFloat16##x##AsUShort##x[] =                        \
+      "intel_convert_bfloat16" #x "_as_ushort" #x;
+_SPIRV_OP()
+_SPIRV_OP(2)
+_SPIRV_OP(3)
+_SPIRV_OP(4)
+_SPIRV_OP(8)
+_SPIRV_OP(16)
+#undef _SPIRV_OP
+#define _SPIRV_OP(x)                                                           \
+  const static char ConvertAsBFloat16##x##Float##x[] =                         \
+      "intel_convert_as_bfloat16" #x "_float" #x;
+_SPIRV_OP()
+_SPIRV_OP(2)
+_SPIRV_OP(3)
+_SPIRV_OP(4)
+_SPIRV_OP(8)
+_SPIRV_OP(16)
+#undef _SPIRV_OP
 } // namespace kOCLBuiltinName
 
 /// Offset for OpenCL image channel order enumeration values.
@@ -303,17 +334,10 @@ const unsigned int OCLImageChannelOrderOffset = 0x10B0;
 const unsigned int OCLImageChannelDataTypeOffset = 0x10D0;
 
 /// OCL 1.x atomic memory order when translated to 2.0 atomics.
-const OCLMemOrderKind OCLLegacyAtomicMemOrder = OCLMO_seq_cst;
+const OCLMemOrderKind OCLLegacyAtomicMemOrder = OCLMO_relaxed;
 
 /// OCL 1.x atomic memory scope when translated to 2.0 atomics.
-const OCLScopeKind OCLLegacyAtomicMemScope = OCLMS_device;
-
-enum IntelFPGAMemoryAccessesVal {
-  BurstCoalesce = 0x1,
-  CacheSizeFlag = 0x2,
-  DontStaticallyCoalesce = 0x4,
-  PrefetchFlag = 0x8
-};
+const OCLScopeKind OCLLegacyAtomicMemScope = OCLMS_work_group;
 
 namespace kOCLVer {
 const unsigned CL12 = 102000;
@@ -350,6 +374,7 @@ enum Kind {
   _SPIRV_OP(cl_khr_mipmap_image_writes)
   _SPIRV_OP(cl_khr_egl_event)
   _SPIRV_OP(cl_khr_srgb_image_writes)
+  _SPIRV_OP(cl_khr_extended_bit_ops)
 #undef _SPIRV_OP
 };
 // clang-format on
@@ -490,19 +515,14 @@ bool isSamplerTy(Type *Ty);
 // Checks if the binary operator is an unfused fmul + fadd instruction.
 bool isUnfusedMulAdd(BinaryOperator *B);
 
-template <typename T> std::string toString(const T *Object) {
-  std::string S;
-  llvm::raw_string_ostream RSOS(S);
-  Object->print(RSOS);
-  RSOS.flush();
-  return S;
-}
-
 // Get data and vector size postfix for sugroup_block_{read|write} builtins
 // as specified by cl_intel_subgroups* extensions.
 // Scalar data assumed to be represented as vector of one element.
 std::string getIntelSubgroupBlockDataPostfix(unsigned ElementBitSize,
                                              unsigned VectorNumElements);
+
+void insertImageNameAccessQualifier(SPIRVAccessQualifierKind Acc,
+                                    std::string &Name);
 } // namespace OCLUtil
 
 using namespace OCLUtil;
