@@ -43,19 +43,21 @@
 #include "llvm/IR/IRBuilder.h"
 #include "llvm/IR/InstVisitor.h"
 #include "llvm/IR/Instructions.h"
+#include "llvm/IR/PassManager.h"
 #include "llvm/Pass.h"
 
 using namespace llvm;
 using namespace SPIRV;
 
 namespace SPIRV {
-class SPIRVLowerBool : public ModulePass, public InstVisitor<SPIRVLowerBool> {
+
+class SPIRVLowerBoolBase : public InstVisitor<SPIRVLowerBoolBase> {
 public:
-  SPIRVLowerBool() : ModulePass(ID), Context(nullptr) {
-    initializeSPIRVLowerBoolPass(*PassRegistry::getPassRegistry());
-  }
+  SPIRVLowerBoolBase() : Context(nullptr) {}
+  virtual ~SPIRVLowerBoolBase() {}
   void replace(Instruction *I, Instruction *NewI) {
     NewI->takeName(I);
+    NewI->setDebugLoc(I->getDebugLoc());
     I->replaceAllUsesWith(NewI);
     I->dropAllReferences();
     I->eraseFromParent();
@@ -72,6 +74,7 @@ public:
       auto Op = I.getOperand(0);
       auto And = BinaryOperator::CreateAnd(
           Op, getScalarOrVectorConstantInt(Op->getType(), 1, false), "", &I);
+      And->setDebugLoc(I.getDebugLoc());
       auto Zero = getScalarOrVectorConstantInt(Op->getType(), 0, false);
       auto Cmp = new ICmpInst(&I, CmpInst::ICMP_NE, And, Zero);
       replace(&I, Cmp);
@@ -101,6 +104,7 @@ public:
       auto One = getScalarOrVectorConstantInt(Ty, 1, false);
       assert(Zero && One && "Couldn't create constant int");
       auto Sel = SelectInst::Create(Op, One, Zero, "", &I);
+      Sel->setDebugLoc(I.getDebugLoc());
       I.setOperand(0, Sel);
     }
   }
@@ -108,7 +112,7 @@ public:
   virtual void visitSExtInst(SExtInst &I) { handleExtInstructions(I); }
   virtual void visitUIToFPInst(UIToFPInst &I) { handleCastInstructions(I); }
   virtual void visitSIToFPInst(SIToFPInst &I) { handleCastInstructions(I); }
-  bool runOnModule(Module &M) override {
+  bool runLowerBool(Module &M) {
     Context = &M.getContext();
     visit(M);
 
@@ -116,16 +120,36 @@ public:
     return true;
   }
 
-  static char ID;
-
 private:
   LLVMContext *Context;
 };
 
-char SPIRVLowerBool::ID = 0;
+class SPIRVLowerBoolPass : public llvm::PassInfoMixin<SPIRVLowerBoolPass>,
+                           public SPIRVLowerBoolBase {
+public:
+  llvm::PreservedAnalyses run(llvm::Module &M,
+                              llvm::ModuleAnalysisManager &MAM) {
+    return runLowerBool(M) ? llvm::PreservedAnalyses::none()
+                           : llvm::PreservedAnalyses::all();
+  }
+};
+
+class SPIRVLowerBoolLegacy : public ModulePass, public SPIRVLowerBoolBase {
+public:
+  SPIRVLowerBoolLegacy() : ModulePass(ID) {
+    initializeSPIRVLowerBoolLegacyPass(*PassRegistry::getPassRegistry());
+  }
+  bool runOnModule(Module &M) override { return runLowerBool(M); }
+
+  static char ID;
+};
+
+char SPIRVLowerBoolLegacy::ID = 0;
 } // namespace SPIRV
 
-INITIALIZE_PASS(SPIRVLowerBool, "spvbool",
+INITIALIZE_PASS(SPIRVLowerBoolLegacy, "spvbool",
                 "Lower instructions with bool operands", false, false)
 
-ModulePass *llvm::createSPIRVLowerBool() { return new SPIRVLowerBool(); }
+ModulePass *llvm::createSPIRVLowerBoolLegacy() {
+  return new SPIRVLowerBoolLegacy();
+}
