@@ -48,6 +48,7 @@
 #include "llvm/IR/InstVisitor.h"
 #include "llvm/IR/IntrinsicInst.h"
 #include "llvm/IR/Module.h"
+#include "llvm/IR/PassManager.h"
 #include "llvm/IRReader/IRReader.h"
 #include "llvm/Linker/Linker.h"
 #include "llvm/Pass.h"
@@ -57,13 +58,12 @@ using namespace llvm;
 using namespace SPIRV;
 
 namespace SPIRV {
-class SPIRVLowerSaddWithOverflow
-    : public ModulePass,
-      public InstVisitor<SPIRVLowerSaddWithOverflow> {
+
+class SPIRVLowerSaddWithOverflowBase
+    : public InstVisitor<SPIRVLowerSaddWithOverflowBase> {
 public:
-  SPIRVLowerSaddWithOverflow() : ModulePass(ID), Context(nullptr) {
-    initializeSPIRVLowerSaddWithOverflowPass(*PassRegistry::getPassRegistry());
-  }
+  SPIRVLowerSaddWithOverflowBase() : Context(nullptr) {}
+  virtual ~SPIRVLowerSaddWithOverflowBase() {}
   virtual void visitIntrinsicInst(CallInst &I) {
     IntrinsicInst *II = dyn_cast<IntrinsicInst>(&I);
     if (!II || II->getIntrinsicID() != Intrinsic::sadd_with_overflow)
@@ -116,7 +116,7 @@ public:
       TheModuleIsModified = true;
   }
 
-  bool runOnModule(Module &M) override {
+  bool runLowerSaddWithOverflow(Module &M) {
     Context = &M.getContext();
     Mod = &M;
     visit(M);
@@ -125,20 +125,43 @@ public:
     return TheModuleIsModified;
   }
 
-  static char ID;
-
 private:
   LLVMContext *Context;
   Module *Mod;
   bool TheModuleIsModified = false;
 };
 
-char SPIRVLowerSaddWithOverflow::ID = 0;
+class SPIRVLowerSaddWithOverflowPass
+    : public llvm::PassInfoMixin<SPIRVLowerSaddWithOverflowPass>,
+      public SPIRVLowerSaddWithOverflowBase {
+public:
+  llvm::PreservedAnalyses run(llvm::Module &M,
+                              llvm::ModuleAnalysisManager &MAM) {
+    return runLowerSaddWithOverflow(M) ? llvm::PreservedAnalyses::none()
+                                       : llvm::PreservedAnalyses::all();
+  }
+};
+
+class SPIRVLowerSaddWithOverflowLegacy : public ModulePass,
+                                         public SPIRVLowerSaddWithOverflowBase {
+public:
+  SPIRVLowerSaddWithOverflowLegacy() : ModulePass(ID) {
+    initializeSPIRVLowerSaddWithOverflowLegacyPass(
+        *PassRegistry::getPassRegistry());
+  }
+
+  bool runOnModule(Module &M) override { return runLowerSaddWithOverflow(M); }
+
+  static char ID;
+};
+
+char SPIRVLowerSaddWithOverflowLegacy::ID = 0;
 } // namespace SPIRV
 
-INITIALIZE_PASS(SPIRVLowerSaddWithOverflow, "spv-lower-llvm_sadd_with_overflow",
+INITIALIZE_PASS(SPIRVLowerSaddWithOverflowLegacy,
+                "spv-lower-llvm_sadd_with_overflow",
                 "Lower llvm.sadd.with.overflow.* intrinsics", false, false)
 
-ModulePass *llvm::createSPIRVLowerSaddWithOverflow() {
-  return new SPIRVLowerSaddWithOverflow();
+ModulePass *llvm::createSPIRVLowerSaddWithOverflowLegacy() {
+  return new SPIRVLowerSaddWithOverflowLegacy();
 }
