@@ -44,9 +44,10 @@
 #include "SPIRVReader.h"
 #include "SPIRVType.h"
 
-#include "llvm/IR/DIBuilder.h"
-#include "llvm/IR/Module.h"
 #include "llvm/ADT/StringExtras.h"
+#include "llvm/IR/DIBuilder.h"
+#include "llvm/IR/IntrinsicInst.h"
+#include "llvm/IR/Module.h"
 
 using namespace std;
 using namespace SPIRVDebug::Operand;
@@ -199,15 +200,46 @@ SPIRVToLLVMDbgTran::transTypeArray(const SPIRVExtInst *DebugInst) {
       transDebugInst<DIType>(BM->get<SPIRVExtInst>(Ops[BaseTypeIdx]));
   size_t TotalCount = 1;
   SmallVector<llvm::Metadata *, 8> Subscripts;
-  for (size_t I = ComponentCountIdx, E = Ops.size(); I < E; ++I) {
-    if (getDbgInst<SPIRVDebug::DebugInfoNone>(Ops[I])) {
-      Subscripts.push_back(Builder.getOrCreateSubrange(1, nullptr));
+  // Ops looks like: { BaseType, count1|upperBound1, count2|upperBound2, ...,
+  // countN|upperBoundN, lowerBound1, lowerBound2, ..., lowerBoundN }
+  for (size_t I = ComponentCountIdx, E = Ops.size() / 2 + 1; I < E; ++I) {
+    if (auto *LocalVar = getDbgInst<SPIRVDebug::LocalVariable>(Ops[I])) {
+      auto *UpperBound = transDebugInst<DILocalVariable>(LocalVar);
+      SPIRVConstant *C = BM->get<SPIRVConstant>(Ops[Ops.size() / 2 + I]);
+      int64_t ConstantAsInt = static_cast<int64_t>(C->getZExtIntValue());
+      auto *LowerBound = ConstantAsMetadata::get(
+          ConstantInt::get(M->getContext(), APInt(64, ConstantAsInt)));
+      Subscripts.push_back(Builder.getOrCreateSubrange(nullptr, LowerBound,
+                                                       UpperBound, nullptr));
+      continue;
+    }
+    if (auto *ExprUB = getDbgInst<SPIRVDebug::Expression>(Ops[I])) {
+      auto *UpperBound = transDebugInst<DIExpression>(ExprUB);
+      auto *ExprLB =
+          getDbgInst<SPIRVDebug::Expression>(Ops[Ops.size() / 2 + I]);
+      auto *LowerBound = transDebugInst<DIExpression>(ExprLB);
+      Subscripts.push_back(Builder.getOrCreateSubrange(nullptr, LowerBound,
+                                                       UpperBound, nullptr));
+      continue;
+    }
+    if (!getDbgInst<SPIRVDebug::DebugInfoNone>(Ops[I])) {
+      SPIRVConstant *C = BM->get<SPIRVConstant>(Ops[I]);
+      int64_t Count = static_cast<int64_t>(C->getZExtIntValue());
+      // If the SPIR-V file was generated by an older version of the translator,
+      // Ops may not contain the LowerBound
+      if (Ops.size() / 2 + I < Ops.size()) {
+        C = BM->get<SPIRVConstant>(Ops[Ops.size() / 2 + I]);
+        int64_t LowerBound = static_cast<int64_t>(C->getZExtIntValue());
+        Subscripts.push_back(Builder.getOrCreateSubrange(LowerBound, Count));
+      } else {
+        auto *CountAsMD = ConstantAsMetadata::get(
+            ConstantInt::get(M->getContext(), APInt(64, Count)));
+        Subscripts.push_back(
+            Builder.getOrCreateSubrange(CountAsMD, nullptr, nullptr, nullptr));
+      }
+      TotalCount *= static_cast<uint64_t>(Count);
       continue;
     }
-    SPIRVConstant *C = BM->get<SPIRVConstant>(Ops[I]);
-    int64_t Count = static_cast<int64_t>(C->getZExtIntValue());
-    Subscripts.push_back(Builder.getOrCreateSubrange(0, Count));
-    TotalCount *= static_cast<uint64_t>(Count);
   }
   DINodeArray SubscriptArray = Builder.getOrCreateArray(Subscripts);
   size_t Size = getDerivedSizeInBits(BaseTy) * TotalCount;
@@ -509,7 +541,8 @@ DINode *SPIRVToLLVMDbgTran::transFunction(const SPIRVExtInst *DebugInst) {
   llvm::DITemplateParameterArray TParamsArray = TParams.get();
 
   DISubprogram *DIS = nullptr;
-  if ((isa<DICompositeType>(Scope) || isa<DINamespace>(Scope)) && !IsDefinition)
+  if (Scope && (isa<DICompositeType>(Scope) || isa<DINamespace>(Scope)) &&
+      !IsDefinition)
     DIS = Builder.createMethod(Scope, Name, LinkageName, File, LineNo, Ty, 0, 0,
                                nullptr, Flags, SPFlags, TParamsArray);
   else
@@ -525,7 +558,7 @@ DINode *SPIRVToLLVMDbgTran::transFunction(const SPIRVExtInst *DebugInst) {
     SPIRVFunction *BF = static_cast<SPIRVFunction *>(E);
     llvm::Function *F = SPIRVReader->transFunction(BF);
     assert(F && "Translation of function failed!");
-    if (!F->hasMetadata())
+    if (!F->hasMetadata("dbg"))
       F->setMetadata("dbg", DIS);
   }
   return DIS;
@@ -634,7 +667,7 @@ MDNode *SPIRVToLLVMDbgTran::transGlobalVariable(const SPIRVExtInst *DebugInst) {
     SPIRVValue *V = BM->get<SPIRVValue>(Ops[VariableIdx]);
     Value *Var = SPIRVReader->transValue(V, nullptr, nullptr);
     llvm::GlobalVariable *GV = dyn_cast_or_null<llvm::GlobalVariable>(Var);
-    if (GV && !GV->hasMetadata())
+    if (GV && !GV->hasMetadata("dbg"))
       GV->addMetadata("dbg", *VarDecl);
   }
   return VarDecl;
@@ -820,7 +853,7 @@ DINode *SPIRVToLLVMDbgTran::transModule(const SPIRVExtInst *DebugInst) {
 
 MDNode *SPIRVToLLVMDbgTran::transExpression(const SPIRVExtInst *DebugInst) {
   const SPIRVWordVec &Args = DebugInst->getArguments();
-  std::vector<int64_t> Ops;
+  std::vector<uint64_t> Ops;
   for (SPIRVId A : Args) {
     SPIRVExtInst *O = BM->get<SPIRVExtInst>(A);
     const SPIRVWordVec &Operands = O->getArguments();
@@ -830,7 +863,7 @@ MDNode *SPIRVToLLVMDbgTran::transExpression(const SPIRVExtInst *DebugInst) {
       Ops.push_back(Operands[I]);
     }
   }
-  ArrayRef<int64_t> Addr(Ops.data(), Ops.size());
+  ArrayRef<uint64_t> Addr(Ops.data(), Ops.size());
   return Builder.createExpression(Addr);
 }
 
@@ -975,9 +1008,16 @@ SPIRVToLLVMDbgTran::transDebugIntrinsic(const SPIRVExtInst *DebugInst,
   case SPIRVDebug::Value: {
     using namespace SPIRVDebug::Operand::DebugValue;
     auto LocalVar = GetLocalVar(Ops[DebugLocalVarIdx]);
-    return Builder.insertDbgValueIntrinsic(
-        GetValue(Ops[ValueIdx]), LocalVar.first,
-        GetExpression(Ops[ExpressionIdx]), LocalVar.second, BB);
+    Value *Val = GetValue(Ops[ValueIdx]);
+    DIExpression *Expr = GetExpression(Ops[ExpressionIdx]);
+    auto *DbgValIntr = Builder.insertDbgValueIntrinsic(
+        Val, LocalVar.first, Expr, LocalVar.second, BB);
+    if (Expr->getNumLocationOperands() == 1) {
+      SmallVector<ValueAsMetadata *, 1> MDs = {ValueAsMetadata::get(Val)};
+      DIArgList *AL = DIArgList::get(M->getContext(), MDs);
+      cast<DbgVariableIntrinsic>(DbgValIntr)->setRawLocation(AL);
+    }
+    return DbgValIntr;
   }
   default:
     llvm_unreachable("Unknown debug intrinsic!");
