@@ -47,7 +47,7 @@
 
 namespace SPIRV {
 
-void SPIRVToOCL::visitCallInst(CallInst &CI) {
+void SPIRVToOCLBase::visitCallInst(CallInst &CI) {
   LLVM_DEBUG(dbgs() << "[visistCallInst] " << CI << '\n');
   auto F = CI.getCalledFunction();
   if (!F)
@@ -82,11 +82,25 @@ void SPIRVToOCL::visitCallInst(CallInst &CI) {
   auto MangledName = F->getName();
   StringRef DemangledName;
   Op OC = OpNop;
+  SPIRVBuiltinVariableKind BuiltinKind = SPIRVBuiltinVariableKind::BuiltInMax;
   if (!oclIsBuiltin(MangledName, DemangledName) ||
-      (OC = getSPIRVFuncOC(DemangledName)) == OpNop)
+      ((OC = getSPIRVFuncOC(DemangledName)) == OpNop &&
+       !getSPIRVBuiltin(DemangledName.str(), BuiltinKind)))
     return;
   LLVM_DEBUG(dbgs() << "DemangledName = " << DemangledName.str() << '\n'
-                    << "OpCode = " << OC << '\n');
+                    << "OpCode = " << OC << '\n'
+                    << "BuiltinKind = " << BuiltinKind << '\n');
+
+  if (BuiltinKind != SPIRVBuiltinVariableKind::BuiltInMax) {
+    if (static_cast<uint32_t>(BuiltinKind) >=
+            internal::BuiltInSubDeviceIDINTEL &&
+        static_cast<uint32_t>(BuiltinKind) <=
+            internal::BuiltInGlobalHWThreadIDINTEL)
+      return;
+
+    visitCallSPIRVBuiltin(&CI, BuiltinKind);
+    return;
+  }
 
   if (OC == OpImageQuerySize || OC == OpImageQuerySizeLod) {
     visitCallSPRIVImageQuerySize(&CI);
@@ -115,6 +129,26 @@ void SPIRVToOCL::visitCallInst(CallInst &CI) {
     visitCallSPIRVImageMediaBlockBuiltin(&CI, OC);
     return;
   }
+  if (isIntelSubgroupOpCode(OC)) {
+    visitCallSPIRVSubgroupINTELBuiltIn(&CI, OC);
+    return;
+  }
+  if (isSubgroupAvcINTELEvaluateOpcode(OC)) {
+    visitCallSPIRVAvcINTELEvaluateBuiltIn(&CI, OC);
+    return;
+  }
+  if (isSubgroupAvcINTELInstructionOpCode(OC)) {
+    visitCallSPIRVAvcINTELInstructionBuiltin(&CI, OC);
+    return;
+  }
+  if (OC == OpBuildNDRange) {
+    visitCallBuildNDRangeBuiltIn(&CI, OC, DemangledName);
+    return;
+  }
+  if (OC == OpGenericCastToPtrExplicit) {
+    visitCallGenericCastToPtrExplicitBuiltIn(&CI, OC);
+    return;
+  }
   if (isCvtOpCode(OC)) {
     visitCallSPIRVCvtBuiltin(&CI, OC, DemangledName);
     return;
@@ -131,11 +165,51 @@ void SPIRVToOCL::visitCallInst(CallInst &CI) {
     visitCallSPIRVImageSampleExplicitLodBuiltIn(&CI, OC);
     return;
   }
+  if (OC == OpImageWrite) {
+    visitCallSPIRVImageWriteBuiltIn(&CI, OC);
+    return;
+  }
+  if (OC == OpImageRead) {
+    visitCallSPIRVImageReadBuiltIn(&CI, OC);
+    return;
+  }
+  if (OC == OpImageQueryOrder || OC == OpImageQueryFormat) {
+    visitCallSPIRVImageQueryBuiltIn(&CI, OC);
+    return;
+  }
+  if (OC == OpEnqueueKernel) {
+    visitCallSPIRVEnqueueKernel(&CI, OC);
+    return;
+  }
+  if (OC == OpGenericPtrMemSemantics) {
+    visitCallSPIRVGenericPtrMemSemantics(&CI);
+    return;
+  }
+  // Check if OC is OpenCL relational builtin except bitselect and select.
+  auto IsOclRelationalOp = [](Op OC) {
+    return isUnaryPredicateOpCode(OC) || OC == OpOrdered || OC == OpUnordered ||
+           OC == OpFOrdEqual || OC == OpFUnordNotEqual ||
+           OC == OpFOrdGreaterThan || OC == OpFOrdGreaterThanEqual ||
+           OC == OpFOrdLessThan || OC == OpFOrdLessThanEqual ||
+           OC == OpFOrdNotEqual;
+  };
+  if (IsOclRelationalOp(OC)) {
+    if (OC == OpAny || OC == OpAll)
+      visitCallSPIRVAnyAll(&CI, OC);
+    else
+      visitCallSPIRVRelational(&CI, OC);
+    return;
+  }
+  if (OC == internal::OpConvertFToBF16INTEL ||
+      OC == internal::OpConvertBF16ToFINTEL) {
+    visitCallSPIRVBFloat16Conversions(&CI, OC);
+    return;
+  }
   if (OCLSPIRVBuiltinMap::rfind(OC))
     visitCallSPIRVBuiltin(&CI, OC);
 }
 
-void SPIRVToOCL::visitCastInst(CastInst &Cast) {
+void SPIRVToOCLBase::visitCastInst(CastInst &Cast) {
   if (!isa<ZExtInst>(Cast) && !isa<SExtInst>(Cast) && !isa<TruncInst>(Cast) &&
       !isa<FPTruncInst>(Cast) && !isa<FPExtInst>(Cast) &&
       !isa<FPToUIInst>(Cast) && !isa<FPToSIInst>(Cast) &&
@@ -170,7 +244,7 @@ void SPIRVToOCL::visitCastInst(CastInst &Cast) {
   Cast.eraseFromParent();
 }
 
-void SPIRVToOCL::visitCallSPRIVImageQuerySize(CallInst *CI) {
+void SPIRVToOCLBase::visitCallSPRIVImageQuerySize(CallInst *CI) {
   Function *Func = CI->getCalledFunction();
   // Get image type
   Type *ArgTy = Func->getFunctionType()->getParamType(0);
@@ -289,7 +363,8 @@ void SPIRVToOCL::visitCallSPRIVImageQuerySize(CallInst *CI) {
   CI->eraseFromParent();
 }
 
-std::string SPIRVToOCL::getUniformArithmeticBuiltinName(CallInst *CI, Op OC) {
+std::string SPIRVToOCLBase::getUniformArithmeticBuiltinName(CallInst *CI,
+                                                            Op OC) {
   assert(isUniformArithmeticOpCode(OC) &&
          "Not intended to handle other than uniform arithmetic opcodes!");
   auto FuncName = OCLSPIRVBuiltinMap::rmap(OC);
@@ -321,8 +396,8 @@ std::string SPIRVToOCL::getUniformArithmeticBuiltinName(CallInst *CI, Op OC) {
   return Prefix + kSPIRVName::GroupPrefix + GroupOp + "_" + Op;
 }
 
-std::string SPIRVToOCL::getNonUniformArithmeticBuiltinName(CallInst *CI,
-                                                           Op OC) {
+std::string SPIRVToOCLBase::getNonUniformArithmeticBuiltinName(CallInst *CI,
+                                                               Op OC) {
   assert(isNonUniformArithmeticOpCode(OC) &&
          "Not intended to handle other than non uniform arithmetic opcodes!");
   std::string Prefix = getGroupBuiltinPrefix(CI);
@@ -375,7 +450,7 @@ std::string SPIRVToOCL::getNonUniformArithmeticBuiltinName(CallInst *CI,
   return Prefix + GroupPrefix + GroupOp + "_" + Op;
 }
 
-std::string SPIRVToOCL::getBallotBuiltinName(CallInst *CI, Op OC) {
+std::string SPIRVToOCLBase::getBallotBuiltinName(CallInst *CI, Op OC) {
   assert((OC == OpGroupNonUniformBallotBitCount) &&
          "Not inteded to handle other opcodes than "
          "OpGroupNonUniformBallotBitCount!");
@@ -402,7 +477,7 @@ std::string SPIRVToOCL::getBallotBuiltinName(CallInst *CI, Op OC) {
   return Prefix + kSPIRVName::GroupPrefix + "ballot_" + GroupOp;
 }
 
-std::string SPIRVToOCL::groupOCToOCLBuiltinName(CallInst *CI, Op OC) {
+std::string SPIRVToOCLBase::groupOCToOCLBuiltinName(CallInst *CI, Op OC) {
   auto FuncName = OCLSPIRVBuiltinMap::rmap(OC);
   assert(FuncName.find(kSPIRVName::GroupPrefix) == 0);
 
@@ -432,7 +507,7 @@ static bool needsInt32RetTy(Op OC) {
          OC == OpGroupNonUniformBallotBitExtract || isGroupLogicalOpCode(OC);
 }
 
-void SPIRVToOCL::visitCallSPIRVGroupBuiltin(CallInst *CI, Op OC) {
+void SPIRVToOCLBase::visitCallSPIRVGroupBuiltin(CallInst *CI, Op OC) {
   auto FuncName = groupOCToOCLBuiltinName(CI, OC);
   auto ModifyArguments = [=](CallInst *, std::vector<Value *> &Args,
                              llvm::Type *&RetTy) {
@@ -472,7 +547,7 @@ void SPIRVToOCL::visitCallSPIRVGroupBuiltin(CallInst *CI, Op OC) {
   mutateCallInstOCL(M, CI, ModifyArguments, ModifyRetTy, &Attrs);
 }
 
-void SPIRVToOCL::visitCallSPIRVPipeBuiltin(CallInst *CI, Op OC) {
+void SPIRVToOCLBase::visitCallSPIRVPipeBuiltin(CallInst *CI, Op OC) {
   auto DemangledName = OCLSPIRVBuiltinMap::rmap(OC);
   bool HasScope = DemangledName.find(kSPIRVName::GroupPrefix) == 0;
   if (HasScope)
@@ -505,7 +580,7 @@ void SPIRVToOCL::visitCallSPIRVPipeBuiltin(CallInst *CI, Op OC) {
       &Attrs);
 }
 
-void SPIRVToOCL::visitCallSPIRVImageMediaBlockBuiltin(CallInst *CI, Op OC) {
+void SPIRVToOCLBase::visitCallSPIRVImageMediaBlockBuiltin(CallInst *CI, Op OC) {
   AttributeList Attrs = CI->getCalledFunction()->getAttributes();
   mutateCallInstOCL(
       M, CI,
@@ -539,9 +614,66 @@ void SPIRVToOCL::visitCallSPIRVImageMediaBlockBuiltin(CallInst *CI, Op OC) {
       },
       &Attrs);
 }
+void SPIRVToOCLBase::visitCallBuildNDRangeBuiltIn(CallInst *CI, Op OC,
+                                                  StringRef DemangledName) {
+  assert(CI->getCalledFunction() && "Unexpected indirect call");
+  AttributeList Attrs = CI->getCalledFunction()->getAttributes();
+  mutateCallInstOCL(
+      M, CI,
+      [=](CallInst *Call, std::vector<Value *> &Args) {
+        assert(Args.size() == 3);
+        // OpenCL built-in has another order of parameters.
+        auto *GlobalWorkSize = Args[0];
+        auto *LocalWorkSize = Args[1];
+        auto *GlobalWorkOffset = Args[2];
+        Args[0] = GlobalWorkOffset;
+        Args[1] = GlobalWorkSize;
+        Args[2] = LocalWorkSize;
+        // __spirv_BuildNDRange_nD, drop __spirv_
+        StringRef S = DemangledName;
+        S = S.drop_front(strlen(kSPIRVName::Prefix));
+        SmallVector<StringRef, 8> Split;
+        // BuildNDRange_nD
+        S.split(Split, kSPIRVPostfix::Divider,
+                /*MaxSplit=*/-1, /*KeepEmpty=*/false);
+        assert(Split.size() >= 2 && "Invalid SPIRV function name");
+        // Cut _nD and add it to function name.
+        return std::string(kOCLBuiltinName::NDRangePrefix) +
+               Split[1].substr(0, 3).str();
+      },
+      &Attrs);
+}
 
-void SPIRVToOCL::visitCallSPIRVCvtBuiltin(CallInst *CI, Op OC,
-                                          StringRef DemangledName) {
+void SPIRVToOCLBase::visitCallGenericCastToPtrExplicitBuiltIn(CallInst *CI,
+                                                              Op OC) {
+  assert(CI->getCalledFunction() && "Unexpected indirect call");
+  AttributeList Attrs = CI->getCalledFunction()->getAttributes();
+  mutateCallInstOCL(
+      M, CI,
+      [=](CallInst *Call, std::vector<Value *> &Args) {
+        auto AddrSpace = static_cast<SPIRAddressSpace>(
+            CI->getType()->getPointerAddressSpace());
+        // The instruction has two arguments, whereas ocl built-in has only one
+        // argument.
+        Args.pop_back();
+        switch (AddrSpace) {
+        case SPIRAS_Global:
+          return std::string(kOCLBuiltinName::ToGlobal);
+        case SPIRAS_Local:
+          return std::string(kOCLBuiltinName::ToLocal);
+        case SPIRAS_Private:
+          return std::string(kOCLBuiltinName::ToPrivate);
+        default:
+          llvm_unreachable("Invalid address space");
+          return std::string();
+        }
+      },
+      &Attrs);
+}
+
+void SPIRVToOCLBase::visitCallSPIRVCvtBuiltin(CallInst *CI, Op OC,
+                                              StringRef DemangledName) {
+  assert(CI->getCalledFunction() && "Unexpected indirect call");
   AttributeList Attrs = CI->getCalledFunction()->getAttributes();
   mutateCallInstOCL(
       M, CI,
@@ -567,7 +699,8 @@ void SPIRVToOCL::visitCallSPIRVCvtBuiltin(CallInst *CI, Op OC,
       &Attrs);
 }
 
-void SPIRVToOCL::visitCallAsyncWorkGroupCopy(CallInst *CI, Op OC) {
+void SPIRVToOCLBase::visitCallAsyncWorkGroupCopy(CallInst *CI, Op OC) {
+  assert(CI->getCalledFunction() && "Unexpected indirect call");
   AttributeList Attrs = CI->getCalledFunction()->getAttributes();
   mutateCallInstOCL(
       M, CI,
@@ -580,7 +713,8 @@ void SPIRVToOCL::visitCallAsyncWorkGroupCopy(CallInst *CI, Op OC) {
       &Attrs);
 }
 
-void SPIRVToOCL::visitCallGroupWaitEvents(CallInst *CI, Op OC) {
+void SPIRVToOCLBase::visitCallGroupWaitEvents(CallInst *CI, Op OC) {
+  assert(CI->getCalledFunction() && "Unexpected indirect call");
   AttributeList Attrs = CI->getCalledFunction()->getAttributes();
   mutateCallInstOCL(
       M, CI,
@@ -608,8 +742,8 @@ static char getTypeSuffix(Type *T) {
 }
 
 // TODO: Handle unsigned integer return type. May need spec change.
-void SPIRVToOCL::visitCallSPIRVImageSampleExplicitLodBuiltIn(CallInst *CI,
-                                                             Op OC) {
+void SPIRVToOCLBase::visitCallSPIRVImageSampleExplicitLodBuiltIn(CallInst *CI,
+                                                                 Op OC) {
   assert(CI->getCalledFunction() && "Unexpected indirect call");
   AttributeList Attrs = CI->getCalledFunction()->getAttributes();
   StringRef ImageTypeName;
@@ -664,7 +798,222 @@ void SPIRVToOCL::visitCallSPIRVImageSampleExplicitLodBuiltIn(CallInst *CI,
   mutateCallInstOCL(M, CI, ModifyArguments, ModifyRetTy, &Attrs);
 }
 
-void SPIRVToOCL::visitCallSPIRVBuiltin(CallInst *CI, Op OC) {
+void SPIRVToOCLBase::visitCallSPIRVImageWriteBuiltIn(CallInst *CI, Op OC) {
+  assert(CI->getCalledFunction() && "Unexpected indirect call");
+  AttributeList Attrs = CI->getCalledFunction()->getAttributes();
+  mutateCallInstOCL(
+      M, CI,
+      [=](CallInst *, std::vector<Value *> &Args) {
+        llvm::Type *T = Args[2]->getType();
+        if (Args.size() > 4) {
+          ConstantInt *ImOp = dyn_cast<ConstantInt>(Args[3]);
+          ConstantFP *LodVal = dyn_cast<ConstantFP>(Args[4]);
+          // Drop "Image Operands" argument.
+          Args.erase(Args.begin() + 3, Args.begin() + 4);
+          // If the image operand is LOD and its value is zero, drop it too.
+          if (ImOp && LodVal && LodVal->isNullValue() &&
+              ImOp->getZExtValue() == ImageOperandsMask::ImageOperandsLodMask)
+            Args.erase(Args.begin() + 3, Args.end());
+          else
+            std::swap(Args[2], Args[3]);
+        }
+        return std::string(kOCLBuiltinName::WriteImage) + getTypeSuffix(T);
+      },
+      &Attrs);
+}
+
+void SPIRVToOCLBase::visitCallSPIRVImageReadBuiltIn(CallInst *CI, Op OC) {
+  assert(CI->getCalledFunction() && "Unexpected indirect call");
+  AttributeList Attrs = CI->getCalledFunction()->getAttributes();
+  mutateCallInstOCL(
+      M, CI,
+      [=](CallInst *, std::vector<Value *> &Args) {
+        // Drop "Image operands" argument
+        if (Args.size() > 2)
+          Args.erase(Args.begin() + 2);
+
+        llvm::Type *T = CI->getType();
+        return std::string(kOCLBuiltinName::ReadImage) + getTypeSuffix(T);
+      },
+      &Attrs);
+}
+
+void SPIRVToOCLBase::visitCallSPIRVImageQueryBuiltIn(CallInst *CI, Op OC) {
+  assert(CI->getCalledFunction() && "Unexpected indirect call");
+  AttributeList Attrs = CI->getCalledFunction()->getAttributes();
+  CI = mutateCallInstOCL(
+      M, CI,
+      [=](CallInst *, std::vector<Value *> &Args) {
+        return OCLSPIRVBuiltinMap::rmap(OC);
+      },
+      &Attrs);
+  unsigned int Offset = 0;
+  if (OC == OpImageQueryFormat)
+    Offset = OCLImageChannelDataTypeOffset;
+  else if (OC == OpImageQueryOrder)
+    Offset = OCLImageChannelOrderOffset;
+  else
+    llvm_unreachable("Unsupported opcode");
+
+  auto *Sub =
+      BinaryOperator::CreateSub(CI, getInt32(M, Offset), "", CI->getNextNode());
+  for (auto &Use : CI->uses()) {
+    if (Use.getUser() == Sub)
+      continue;
+    Use.set(Sub);
+  }
+}
+
+void SPIRVToOCLBase::visitCallSPIRVSubgroupINTELBuiltIn(CallInst *CI, Op OC) {
+  assert(CI->getCalledFunction() && "Unexpected indirect call");
+  AttributeList Attrs = CI->getCalledFunction()->getAttributes();
+  mutateCallInstOCL(
+      M, CI,
+      [=](CallInst *, std::vector<Value *> &Args) {
+        std::stringstream Name;
+        Type *DataTy = nullptr;
+        switch (OC) {
+        case OpSubgroupBlockReadINTEL:
+        case OpSubgroupImageBlockReadINTEL:
+          Name << "intel_sub_group_block_read";
+          DataTy = CI->getType();
+          break;
+        case OpSubgroupBlockWriteINTEL:
+          Name << "intel_sub_group_block_write";
+          DataTy = CI->getOperand(1)->getType();
+          break;
+        case OpSubgroupImageBlockWriteINTEL:
+          Name << "intel_sub_group_block_write";
+          DataTy = CI->getOperand(2)->getType();
+          break;
+        default:
+          return OCLSPIRVBuiltinMap::rmap(OC);
+        }
+        assert(DataTy && "Intel subgroup block builtins should have data type");
+        unsigned VectorNumElements = 1;
+        if (FixedVectorType *VT = dyn_cast<FixedVectorType>(DataTy))
+          VectorNumElements = VT->getNumElements();
+        unsigned ElementBitSize = DataTy->getScalarSizeInBits();
+        Name << getIntelSubgroupBlockDataPostfix(ElementBitSize,
+                                                 VectorNumElements);
+        return Name.str();
+      },
+      &Attrs);
+}
+
+void SPIRVToOCLBase::visitCallSPIRVAvcINTELEvaluateBuiltIn(CallInst *CI,
+                                                           Op OC) {
+  assert(CI->getCalledFunction() && "Unexpected indirect call");
+  AttributeList Attrs = CI->getCalledFunction()->getAttributes();
+  mutateCallInstOCL(
+      M, CI,
+      [=](CallInst *, std::vector<Value *> &Args) {
+        // There are three types of AVC Intel Evaluate opcodes:
+        // 1. With multi reference images - does not use OpVmeImageINTEL opcode
+        // for reference images
+        // 2. With dual reference images - uses two OpVmeImageINTEL opcodes for
+        // reference image
+        // 3. With single reference image - uses one OpVmeImageINTEL opcode for
+        // reference image
+        int NumImages = std::count_if(Args.begin(), Args.end(), [](Value *Arg) {
+          if (auto *PT = dyn_cast<PointerType>(Arg->getType())) {
+            if (auto *ST = dyn_cast<StructType>(PT->getPointerElementType())) {
+              if (ST->getName().startswith("spirv.VmeImageINTEL"))
+                return true;
+            }
+          }
+          return false;
+        });
+        auto EraseVmeImageCall = [](CallInst *CI) {
+          if (CI->hasOneUse()) {
+            CI->replaceAllUsesWith(UndefValue::get(CI->getType()));
+            CI->dropAllReferences();
+            CI->eraseFromParent();
+          }
+        };
+        if (NumImages) {
+          CallInst *SrcImage = cast<CallInst>(Args[0]);
+          if (NumImages == 1) {
+            // Multi reference opcode - remove src image OpVmeImageINTEL opcode
+            // and replace it with corresponding OpImage and OpSampler arguments
+            size_t SamplerPos = Args.size() - 1;
+            Args.erase(Args.begin(), Args.begin() + 1);
+            Args.insert(Args.begin(), SrcImage->getOperand(0));
+            Args.insert(Args.begin() + SamplerPos, SrcImage->getOperand(1));
+            EraseVmeImageCall(SrcImage);
+          } else {
+            CallInst *FwdRefImage = cast<CallInst>(Args[1]);
+            CallInst *BwdRefImage =
+                NumImages == 3 ? cast<CallInst>(Args[2]) : nullptr;
+            // Single reference opcode - remove src and ref image
+            // OpVmeImageINTEL opcodes and replace them with src and ref OpImage
+            // opcodes and OpSampler
+            Args.erase(Args.begin(), Args.begin() + NumImages);
+            // insert source OpImage and OpSampler
+            auto SrcOps = SrcImage->args();
+            Args.insert(Args.begin(), SrcOps.begin(), SrcOps.end());
+            // insert reference OpImage
+            Args.insert(Args.begin() + 1, FwdRefImage->getOperand(0));
+            EraseVmeImageCall(SrcImage);
+            EraseVmeImageCall(FwdRefImage);
+            if (BwdRefImage) {
+              // Dual reference opcode - insert second reference OpImage
+              // argument
+              Args.insert(Args.begin() + 2, BwdRefImage->getOperand(0));
+              EraseVmeImageCall(BwdRefImage);
+            }
+          }
+        } else
+          llvm_unreachable("invalid avc instruction");
+
+        return OCLSPIRVSubgroupAVCIntelBuiltinMap::rmap(OC);
+      },
+      &Attrs);
+}
+
+void SPIRVToOCLBase::visitCallSPIRVGenericPtrMemSemantics(CallInst *CI) {
+  AttributeList Attrs = CI->getCalledFunction()->getAttributes();
+  mutateCallInstOCL(
+      M, CI,
+      [=](CallInst *, std::vector<Value *> &Args, Type *&RetTy) {
+        return OCLSPIRVBuiltinMap::rmap(OpGenericPtrMemSemantics);
+      },
+      [=](CallInst *CI) -> Instruction * {
+        auto *Shl = BinaryOperator::CreateShl(CI, getInt32(M, 8), "");
+        Shl->insertAfter(CI);
+        return Shl;
+      },
+      &Attrs);
+}
+
+void SPIRVToOCLBase::visitCallSPIRVBFloat16Conversions(CallInst *CI, Op OC) {
+  AttributeList Attrs = CI->getCalledFunction()->getAttributes();
+  mutateCallInstOCL(
+      M, CI,
+      [=](CallInst *, std::vector<Value *> &Args) {
+        Type *ArgTy = CI->getOperand(0)->getType();
+        std::string N =
+            ArgTy->isVectorTy()
+                ? std::to_string(cast<FixedVectorType>(ArgTy)->getNumElements())
+                : "";
+        std::string Name;
+        switch (static_cast<uint32_t>(OC)) {
+        case internal::OpConvertFToBF16INTEL:
+          Name = "intel_convert_bfloat16" + N + "_as_ushort" + N;
+          break;
+        case internal::OpConvertBF16ToFINTEL:
+          Name = "intel_convert_as_bfloat16" + N + "_float" + N;
+          break;
+        default:
+          break; // do nothing
+        }
+        return Name;
+      },
+      &Attrs);
+}
+
+void SPIRVToOCLBase::visitCallSPIRVBuiltin(CallInst *CI, Op OC) {
+  assert(CI->getCalledFunction() && "Unexpected indirect call");
   AttributeList Attrs = CI->getCalledFunction()->getAttributes();
   mutateCallInstOCL(
       M, CI,
@@ -674,7 +1023,32 @@ void SPIRVToOCL::visitCallSPIRVBuiltin(CallInst *CI, Op OC) {
       &Attrs);
 }
 
-void SPIRVToOCL::visitCallSPIRVOCLExt(CallInst *CI, OCLExtOpKind Kind) {
+void SPIRVToOCLBase::visitCallSPIRVBuiltin(CallInst *CI,
+                                           SPIRVBuiltinVariableKind Kind) {
+  assert(CI->getCalledFunction() && "Unexpected indirect call");
+  AttributeList Attrs = CI->getCalledFunction()->getAttributes();
+  mutateCallInstOCL(
+      M, CI,
+      [=](CallInst *, std::vector<Value *> &Args) {
+        return SPIRSPIRVBuiltinVariableMap::rmap(Kind);
+      },
+      &Attrs);
+}
+
+void SPIRVToOCLBase::visitCallSPIRVAvcINTELInstructionBuiltin(CallInst *CI,
+                                                              Op OC) {
+  assert(CI->getCalledFunction() && "Unexpected indirect call");
+  AttributeList Attrs = CI->getCalledFunction()->getAttributes();
+  mutateCallInstOCL(
+      M, CI,
+      [=](CallInst *, std::vector<Value *> &Args) {
+        return OCLSPIRVSubgroupAVCIntelBuiltinMap::rmap(OC);
+      },
+      &Attrs);
+}
+
+void SPIRVToOCLBase::visitCallSPIRVOCLExt(CallInst *CI, OCLExtOpKind Kind) {
+  assert(CI->getCalledFunction() && "Unexpected indirect call");
   AttributeList Attrs = CI->getCalledFunction()->getAttributes();
   mutateCallInstOCL(
       M, CI,
@@ -684,7 +1058,8 @@ void SPIRVToOCL::visitCallSPIRVOCLExt(CallInst *CI, OCLExtOpKind Kind) {
       &Attrs);
 }
 
-void SPIRVToOCL::visitCallSPIRVVLoadn(CallInst *CI, OCLExtOpKind Kind) {
+void SPIRVToOCLBase::visitCallSPIRVVLoadn(CallInst *CI, OCLExtOpKind Kind) {
+  assert(CI->getCalledFunction() && "Unexpected indirect call");
   AttributeList Attrs = CI->getCalledFunction()->getAttributes();
   mutateCallInstOCL(
       M, CI,
@@ -702,7 +1077,8 @@ void SPIRVToOCL::visitCallSPIRVVLoadn(CallInst *CI, OCLExtOpKind Kind) {
       &Attrs);
 }
 
-void SPIRVToOCL::visitCallSPIRVVStore(CallInst *CI, OCLExtOpKind Kind) {
+void SPIRVToOCLBase::visitCallSPIRVVStore(CallInst *CI, OCLExtOpKind Kind) {
+  assert(CI->getCalledFunction() && "Unexpected indirect call");
   AttributeList Attrs = CI->getCalledFunction()->getAttributes();
   mutateCallInstOCL(
       M, CI,
@@ -725,7 +1101,7 @@ void SPIRVToOCL::visitCallSPIRVVStore(CallInst *CI, OCLExtOpKind Kind) {
             Kind == OpenCLLIB::Vstorea_halfn ||
             Kind == OpenCLLIB::Vstorea_halfn_r || Kind == OpenCLLIB::Vstoren) {
           if (auto DataType = dyn_cast<VectorType>(Args[0]->getType())) {
-            uint64_t NumElements = DataType->getElementCount().getValue();
+            uint64_t NumElements = DataType->getElementCount().getFixedValue();
             assert((NumElements == 2 || NumElements == 3 || NumElements == 4 ||
                     NumElements == 8 || NumElements == 16) &&
                    "Unsupported vector size for vstore instruction!");
@@ -740,7 +1116,8 @@ void SPIRVToOCL::visitCallSPIRVVStore(CallInst *CI, OCLExtOpKind Kind) {
       &Attrs);
 }
 
-void SPIRVToOCL::visitCallSPIRVPrintf(CallInst *CI, OCLExtOpKind Kind) {
+void SPIRVToOCLBase::visitCallSPIRVPrintf(CallInst *CI, OCLExtOpKind Kind) {
+  assert(CI->getCalledFunction() && "Unexpected indirect call");
   AttributeList Attrs = CI->getCalledFunction()->getAttributes();
   CallInst *NewCI = mutateCallInstOCL(
       M, CI,
@@ -757,7 +1134,58 @@ void SPIRVToOCL::visitCallSPIRVPrintf(CallInst *CI, OCLExtOpKind Kind) {
     NewCI->getCalledFunction()->setName(TargetName);
 }
 
-std::string SPIRVToOCL::getGroupBuiltinPrefix(CallInst *CI) {
+void SPIRVToOCLBase::visitCallSPIRVAnyAll(CallInst *CI, Op OC) {
+  AttributeList Attrs = CI->getCalledFunction()->getAttributes();
+  mutateCallInstOCL(
+      M, CI,
+      [=](CallInst *, std::vector<Value *> &Args, Type *&RetTy) {
+        Type *Int8Ty = Type::getInt8Ty(*Ctx);
+        auto *OldArg = CI->getOperand(0);
+        auto *OldArgTy = cast<FixedVectorType>(OldArg->getType());
+        if (Int8Ty != OldArgTy->getElementType()) {
+          auto *NewArgTy =
+              FixedVectorType::get(Int8Ty, OldArgTy->getNumElements());
+          auto *NewArg =
+              CastInst::CreateSExtOrBitCast(OldArg, NewArgTy, "", CI);
+          Args[0] = NewArg;
+        }
+        RetTy = Type::getInt32Ty(*Ctx);
+        return OCLSPIRVBuiltinMap::rmap(OC);
+      },
+      [=](CallInst *NewCI) -> Instruction * {
+        return CastInst::CreateTruncOrBitCast(NewCI, CI->getType(), "",
+                                              NewCI->getNextNode());
+      },
+      &Attrs);
+}
+
+void SPIRVToOCLBase::visitCallSPIRVRelational(CallInst *CI, Op OC) {
+  AttributeList Attrs = CI->getCalledFunction()->getAttributes();
+  mutateCallInstOCL(
+      M, CI,
+      [=](CallInst *, std::vector<Value *> & /*Args*/, Type *&RetTy) {
+        Type *IntTy = Type::getInt32Ty(*Ctx);
+        RetTy = IntTy;
+        if (CI->getType()->isVectorTy()) {
+          auto *OpElemTy = cast<FixedVectorType>(CI->getOperand(0)->getType())
+                               ->getElementType();
+          if (OpElemTy->isDoubleTy())
+            IntTy = Type::getInt64Ty(*Ctx);
+          if (OpElemTy->isHalfTy())
+            IntTy = Type::getInt16Ty(*Ctx);
+          RetTy = FixedVectorType::get(
+              IntTy, cast<FixedVectorType>(CI->getType())->getNumElements());
+        }
+        return OCLSPIRVBuiltinMap::rmap(OC);
+      },
+      [=](CallInst *NewCI) -> Instruction * {
+        return CastInst::CreateTruncOrBitCast(NewCI, CI->getType(), "",
+                                              NewCI->getNextNode());
+      },
+      &Attrs);
+}
+
+std::string SPIRVToOCLBase::getGroupBuiltinPrefix(CallInst *CI) {
   std::string Prefix;
   auto ES = getArgAsScope(CI, 0);
   switch (ES) {
@@ -773,6 +1201,64 @@ std::string SPIRVToOCL::getGroupBuiltinPrefix(CallInst *CI) {
   return Prefix;
 }
 
+std::string
+SPIRVToOCLBase::getOCLImageOpaqueType(SmallVector<std::string, 8> &Postfixes) {
+  SmallVector<int, 7> Ops;
+  for (unsigned I = 1; I < 8; ++I)
+    Ops.push_back(atoi(Postfixes[I].c_str()));
+  SPIRVTypeImageDescriptor Desc(static_cast<SPIRVImageDimKind>(Ops[0]), Ops[1],
+                                Ops[2], Ops[3], Ops[4], Ops[5]);
+
+  std::string OCLStructName =
+      std::string(kSPR2TypeName::OCLPrefix) + rmap<std::string>(Desc);
+
+  SPIRVAccessQualifierKind Acc = static_cast<SPIRVAccessQualifierKind>(Ops[6]);
+  insertImageNameAccessQualifier(Acc, OCLStructName);
+  return OCLStructName;
+}
+
+std::string
+SPIRVToOCLBase::getOCLPipeOpaqueType(SmallVector<std::string, 8> &Postfixes) {
+  assert(Postfixes.size() == 1);
+  unsigned PipeAccess = atoi(Postfixes[0].c_str());
+  assert((PipeAccess == AccessQualifierReadOnly ||
+          PipeAccess == AccessQualifierWriteOnly) &&
+         "Invalid access qualifier");
+  return PipeAccess ? kSPR2TypeName::PipeWO : kSPR2TypeName::PipeRO;
+}
+
+void SPIRVToOCLBase::translateOpaqueTypes() {
+  for (auto *S : M->getIdentifiedStructTypes()) {
+    StringRef STName = S->getStructName();
+    bool IsSPIRVOpaque =
+        S->isOpaque() && STName.startswith(kSPIRVTypeName::PrefixAndDelim);
+
+    if (!IsSPIRVOpaque)
+      continue;
+
+    SmallVector<std::string, 8> Postfixes;
+    std::string DecodedST = decodeSPIRVTypeName(STName, Postfixes);
+
+    if (!SPIRVOpaqueTypeOpCodeMap::find(DecodedST))
+      continue;
+
+    Op OP = SPIRVOpaqueTypeOpCodeMap::map(DecodedST);
+    std::string OCLOpaqueName;
+    if (OP == OpTypeImage)
+      OCLOpaqueName = getOCLImageOpaqueType(Postfixes);
+    else if (OP == OpTypePipe)
+      OCLOpaqueName = getOCLPipeOpaqueType(Postfixes);
+    else if (isSubgroupAvcINTELTypeOpCode(OP))
+      OCLOpaqueName = OCLSubgroupINTELTypeOpCodeMap::rmap(OP);
+    else if (isOpaqueGenericTypeOpCode(OP))
+      OCLOpaqueName = OCLOpaqueTypeOpCodeMap::rmap(OP);
+    else
+      continue;
+
+    S->setName(OCLOpaqueName);
+  }
+}
+
 } // namespace SPIRV
 
 ModulePass *
@@ -780,14 +1266,13 @@ llvm::createSPIRVBIsLoweringPass(Module &M,
                                  SPIRV::BIsRepresentation BIsRepresentation) {
   switch (BIsRepresentation) {
   case SPIRV::BIsRepresentation::OpenCL12:
-    return createSPIRVToOCL12();
+    return createSPIRVToOCL12Legacy();
   case SPIRV::BIsRepresentation::OpenCL20:
-    return createSPIRVToOCL20();
+    return createSPIRVToOCL20Legacy();
   case SPIRV::BIsRepresentation::SPIRVFriendlyIR:
     // nothing to do, already done
     return nullptr;
-  default:
-    llvm_unreachable("Unsupported built-ins representation");
-    return nullptr;
   }
+  llvm_unreachable("Unsupported built-ins representation");
+  return nullptr;
 }
