@@ -42,7 +42,7 @@
 
 #include "SPIRVInternal.h"
 #include "SPIRVUtil.h"
-#include "spirv.hpp"
+#include "spirv/unified1/spirv.hpp"
 
 namespace VectorComputeUtil {
 
@@ -111,6 +111,7 @@ const static char VCSIMTCall[] = "VCSIMTCall";
 const static char VCCallable[] = "VCCallable";
 const static char VCSingleElementVector[] = "VCSingleElementVector";
 const static char VCFCEntry[] = "VCFCEntry";
+const static char VCMediaBlockIO[] = "VCMediaBlockIO";
 } // namespace kVCMetadata
 
 namespace kVCType {
