@@ -44,6 +44,7 @@
 #include "SPIRVDecorate.h"
 #include "SPIRVFunction.h"
 #include "SPIRVInstruction.h"
+#include "SPIRVMemAliasingINTEL.h"
 #include "SPIRVStream.h"
 #include "SPIRVType.h"
 
@@ -253,7 +254,7 @@ void SPIRVEntry::validateBuiltin(SPIRVWord TheSet, SPIRVWord Index) const {
 
 void SPIRVEntry::addDecorate(SPIRVDecorate *Dec) {
   auto Kind = Dec->getDecorateKind();
-  Decorates.insert(std::make_pair(Dec->getDecorateKind(), Dec));
+  Decorates.insert(std::make_pair(Kind, Dec));
   Module->addDecorate(Dec);
   if (Kind == spv::DecorationLinkageAttributes) {
     auto *LinkageAttr = static_cast<const SPIRVDecorateLinkageAttr *>(Dec);
@@ -262,12 +263,25 @@ void SPIRVEntry::addDecorate(SPIRVDecorate *Dec) {
   SPIRVDBG(spvdbgs() << "[addDecorate] " << *Dec << '\n';)
 }
 
+void SPIRVEntry::addDecorate(SPIRVDecorateId *Dec) {
+  DecorateIds.insert(std::make_pair(Dec->getDecorateKind(), Dec));
+  Module->addDecorate(Dec);
+  SPIRVDBG(spvdbgs() << "[addDecorateId] " << *Dec << '\n';)
+}
+
 void SPIRVEntry::addDecorate(Decoration Kind) {
   addDecorate(new SPIRVDecorate(Kind, this));
 }
 
 void SPIRVEntry::addDecorate(Decoration Kind, SPIRVWord Literal) {
-  addDecorate(new SPIRVDecorate(Kind, this, Literal));
+  switch (static_cast<int>(Kind)) {
+  case internal::DecorationAliasScopeINTEL:
+  case internal::DecorationNoAliasINTEL:
+    addDecorate(new SPIRVDecorateId(Kind, this, Literal));
+    return;
+  default:
+    addDecorate(new SPIRVDecorate(Kind, this, Literal));
+  }
 }
 
 void SPIRVEntry::eraseDecorate(Decoration Dec) { Decorates.erase(Dec); }
@@ -277,6 +291,11 @@ void SPIRVEntry::takeDecorates(SPIRVEntry *E) {
   SPIRVDBG(spvdbgs() << "[takeDecorates] " << Id << '\n';)
 }
 
+void SPIRVEntry::takeDecorateIds(SPIRVEntry *E) {
+  DecorateIds = std::move(E->DecorateIds);
+  SPIRVDBG(spvdbgs() << "[takeDecorateIds] " << Id << '\n';)
+}
+
 void SPIRVEntry::setLine(const std::shared_ptr<const SPIRVLine> &L) {
   Line = L;
   SPIRVDBG(if (L) spvdbgs() << "[setLine] " << *L << '\n';)
@@ -311,6 +330,7 @@ void SPIRVEntry::takeMemberDecorates(SPIRVEntry *E) {
 void SPIRVEntry::takeAnnotations(SPIRVForward *E) {
   Module->setName(this, E->getName());
   takeDecorates(E);
+  takeDecorateIds(E);
   takeMemberDecorates(E);
   if (OpCode == OpFunction)
     static_cast<SPIRVFunction *>(this)->takeExecutionModes(E);
@@ -320,7 +340,7 @@ void SPIRVEntry::takeAnnotations(SPIRVForward *E) {
 // first decoration of such kind at Index.
 bool SPIRVEntry::hasDecorate(Decoration Kind, size_t Index,
                              SPIRVWord *Result) const {
-  DecorateMapType::const_iterator Loc = Decorates.find(Kind);
+  auto Loc = Decorates.find(Kind);
   if (Loc == Decorates.end())
     return false;
   if (Result)
@@ -328,6 +348,16 @@ bool SPIRVEntry::hasDecorate(Decoration Kind, size_t Index,
   return true;
 }
 
+bool SPIRVEntry::hasDecorateId(Decoration Kind, size_t Index,
+                               SPIRVId *Result) const {
+  auto Loc = DecorateIds.find(Kind);
+  if (Loc == DecorateIds.end())
+    return false;
+  if (Result)
+    *Result = Loc->second->getLiteral(Index);
+  return true;
+}
+
 // Check if an entry member has Kind of decoration and get the literal of the
 // first decoration of such kind at Index.
 bool SPIRVEntry::hasMemberDecorate(Decoration Kind, size_t Index,
@@ -369,6 +399,15 @@ SPIRVEntry::getDecorationLiterals(Decoration Kind) const {
   return (Loc->second->getVecLiteral());
 }
 
+std::vector<SPIRVId>
+SPIRVEntry::getDecorationIdLiterals(Decoration Kind) const {
+  auto Loc = DecorateIds.find(Kind);
+  if (Loc == DecorateIds.end())
+    return {};
+
+  return (Loc->second->getVecLiteral());
+}
+
 std::vector<SPIRVWord>
 SPIRVEntry::getMemberDecorationLiterals(Decoration Kind,
                                         SPIRVWord MemberNumber) const {
@@ -402,6 +441,36 @@ SPIRVEntry::getDecorations(Decoration Kind) const {
   return Decors;
 }
 
+std::vector<SPIRVDecorate const *> SPIRVEntry::getDecorations() const {
+  std::vector<SPIRVDecorate const *> Decors;
+  Decors.reserve(Decorates.size());
+  for (auto &DecoPair : Decorates)
+    Decors.push_back(DecoPair.second);
+  return Decors;
+}
+
+std::set<SPIRVId> SPIRVEntry::getDecorateId(Decoration Kind,
+                                            size_t Index) const {
+  auto Range = DecorateIds.equal_range(Kind);
+  std::set<SPIRVId> Value;
+  for (auto I = Range.first, E = Range.second; I != E; ++I) {
+    assert(Index < I->second->getLiteralCount() && "Invalid index");
+    Value.insert(I->second->getLiteral(Index));
+  }
+  return Value;
+}
+
+std::vector<SPIRVDecorateId const *>
+SPIRVEntry::getDecorationIds(Decoration Kind) const {
+  auto Range = DecorateIds.equal_range(Kind);
+  std::vector<SPIRVDecorateId const *> Decors;
+  Decors.reserve(DecorateIds.count(Kind));
+  for (auto I = Range.first, E = Range.second; I != E; ++I) {
+    Decors.push_back(I->second);
+  }
+  return Decors;
+}
+
 bool SPIRVEntry::hasLinkageType() const {
   return OpCode == OpFunction || OpCode == OpVariable;
 }
@@ -428,6 +497,8 @@ bool SPIRVEntry::isExtInst(const SPIRVExtInstSetKind InstSet,
 void SPIRVEntry::encodeDecorate(spv_ostream &O) const {
   for (auto &I : Decorates)
     O << *I.second;
+  for (auto &I : DecorateIds)
+    O << *I.second;
 }
 
 SPIRVLinkageTypeKind SPIRVEntry::getLinkageType() const {
@@ -478,9 +549,11 @@ void SPIRVEntryPoint::encode(spv_ostream &O) const {
 }
 
 void SPIRVEntryPoint::decode(std::istream &I) {
-  getDecoder(I) >> ExecModel >> Target >> Name >> Variables;
+  getDecoder(I) >> ExecModel >> Target >> Name;
+  Variables.resize(WordCount - FixedWC - getSizeInWords(Name) + 1);
+  getDecoder(I) >> Variables;
   Module->setName(getOrCreateTarget(), Name);
-  Module->addEntryPoint(ExecModel, Target);
+  Module->addEntryPoint(ExecModel, Target, Name, Variables);
 }
 
 void SPIRVExecutionMode::encode(spv_ostream &O) const {
@@ -489,7 +562,7 @@ void SPIRVExecutionMode::encode(spv_ostream &O) const {
 
 void SPIRVExecutionMode::decode(std::istream &I) {
   getDecoder(I) >> Target >> ExecMode;
-  switch (ExecMode) {
+  switch (static_cast<uint32_t>(ExecMode)) {
   case ExecutionModeLocalSize:
   case ExecutionModeLocalSizeHint:
   case ExecutionModeMaxWorkgroupSizeINTEL:
@@ -512,6 +585,7 @@ void SPIRVExecutionMode::decode(std::istream &I) {
   case ExecutionModeMaxWorkDimINTEL:
   case ExecutionModeNumSIMDWorkitemsINTEL:
   case ExecutionModeSchedulerTargetFmaxMhzINTEL:
+  case internal::ExecutionModeStreamingInterfaceINTEL:
     WordLiterals.resize(1);
     break;
   default:
