@@ -47,7 +47,7 @@
 #define SPIRV_LIBSPIRV_SPIRVNAMEMAPENUM_H
 
 #include "SPIRVEnum.h"
-#include "spirv.hpp"
+#include "spirv/unified1/spirv.hpp"
 #include "spirv_internal.hpp"
 
 using namespace spv;
@@ -57,6 +57,7 @@ namespace SPIRV {
 template <> inline void SPIRVMap<LinkageType, std::string>::init() {
   add(LinkageTypeExport, "Export");
   add(LinkageTypeImport, "Import");
+  add(LinkageTypeLinkOnceODR, "LinkOnceODR");
   add(internal::LinkageTypeInternal, "Internal");
   add(LinkageTypeMax, "Max");
 }
@@ -129,6 +130,7 @@ template <> inline void SPIRVMap<Decoration, std::string>::init() {
   add(DecorationAliasedPointerEXT, "AliasedPointerEXT");
   add(DecorationSIMTCallINTEL, "SIMTCallINTEL");
   add(DecorationReferencedIndirectlyINTEL, "ReferencedIndirectlyINTEL");
+  add(DecorationClobberINTEL, "ClobberINTEL");
   add(DecorationSideEffectsINTEL, "SideEffectsINTEL");
   add(DecorationVectorComputeVariableINTEL, "VectorComputeVariableINTEL");
   add(DecorationFuncParamIOKindINTEL, "FuncParamIOKind");
@@ -158,14 +160,31 @@ template <> inline void SPIRVMap<Decoration, std::string>::init() {
   add(DecorationCacheSizeINTEL, "CacheSizeINTEL");
   add(DecorationDontStaticallyCoalesceINTEL, "DontStaticallyCoalesceINTEL");
   add(DecorationPrefetchINTEL, "PrefetchINTEL");
+  add(DecorationStallEnableINTEL, "StallEnableINTEL");
+  add(DecorationFuseLoopsInFunctionINTEL, "FuseLoopsInFunctionINTEL");
   add(DecorationBufferLocationINTEL, "BufferLocationINTEL");
   add(DecorationIOPipeStorageINTEL, "IOPipeStorageINTEL");
   add(DecorationFunctionFloatingPointModeINTEL,
       "FunctionFloatingPointModeINTEL");
   add(DecorationSingleElementVectorINTEL, "SingleElementVectorINTEL");
-  add(DecorationCallableFunctionINTEL, "CallableFunctionINTEL");
-  add(DecorationStallEnableINTEL, "StallEnableINTEL");
-  add(DecorationFuseLoopsInFunctionINTEL, "FuseLoopsInFunctionINTEL");
+  add(DecorationVectorComputeCallableFunctionINTEL,
+      "VectorComputeCallableFunctionINTEL");
+  add(DecorationMediaBlockIOINTEL, "MediaBlockIOINTEL");
+
+  // From spirv_internal.hpp
+  add(internal::DecorationCallableFunctionINTEL, "CallableFunctionINTEL");
+  add(internal::DecorationMathOpDSPModeINTEL, "MathOpDSPModeINTEL");
+  add(internal::DecorationAliasScopeINTEL, "AliasScopeINTEL");
+  add(internal::DecorationNoAliasINTEL, "NoAliasINTEL");
+  add(internal::DecorationInitiationIntervalINTEL, "InitiationIntervalINTEL");
+  add(internal::DecorationMaxConcurrencyINTEL, "MaxConcurrencyINTEL");
+  add(internal::DecorationPipelineEnableINTEL, "PipelineEnableINTEL");
+  add(internal::DecorationRuntimeAlignedINTEL, "RuntimeAlignedINTEL");
+  add(internal::DecorationHostAccessINTEL, "HostAccessINTEL");
+  add(internal::DecorationInitModeINTEL, "InitModeINTEL");
+  add(internal::DecorationImplementInCSRINTEL, "ImplementInCSRINTEL");
+  add(internal::DecorationArgumentAttributeINTEL, "ArgumentAttributeINTEL");
+
   add(DecorationMax, "Max");
 }
 SPIRV_DEF_NAMEMAP(Decoration, SPIRVDecorationNameMap)
@@ -225,8 +244,10 @@ template <> inline void SPIRVMap<BuiltIn, std::string>::init() {
   add(BuiltInBaseVertex, "BuiltInBaseVertex");
   add(BuiltInBaseInstance, "BuiltInBaseInstance");
   add(BuiltInDrawIndex, "BuiltInDrawIndex");
+  add(BuiltInPrimitiveShadingRateKHR, "BuiltInPrimitiveShadingRateKHR");
   add(BuiltInDeviceIndex, "BuiltInDeviceIndex");
   add(BuiltInViewIndex, "BuiltInViewIndex");
+  add(BuiltInShadingRateKHR, "BuiltInShadingRateKHR");
   add(BuiltInBaryCoordNoPerspAMD, "BuiltInBaryCoordNoPerspAMD");
   add(BuiltInBaryCoordNoPerspCentroidAMD, "BuiltInBaryCoordNoPerspCentroidAMD");
   add(BuiltInBaryCoordNoPerspSampleAMD, "BuiltInBaryCoordNoPerspSampleAMD");
@@ -255,25 +276,41 @@ template <> inline void SPIRVMap<BuiltIn, std::string>::init() {
   add(BuiltInFragmentSizeNV, "BuiltInFragmentSizeNV");
   add(BuiltInFragInvocationCountEXT, "BuiltInFragInvocationCountEXT");
   add(BuiltInInvocationsPerPixelNV, "BuiltInInvocationsPerPixelNV");
+  add(BuiltInLaunchIdKHR, "BuiltInLaunchIdKHR");
   add(BuiltInLaunchIdNV, "BuiltInLaunchIdNV");
+  add(BuiltInLaunchSizeKHR, "BuiltInLaunchSizeKHR");
   add(BuiltInLaunchSizeNV, "BuiltInLaunchSizeNV");
+  add(BuiltInWorldRayOriginKHR, "BuiltInWorldRayOriginKHR");
   add(BuiltInWorldRayOriginNV, "BuiltInWorldRayOriginNV");
+  add(BuiltInWorldRayDirectionKHR, "BuiltInWorldRayDirectionKHR");
   add(BuiltInWorldRayDirectionNV, "BuiltInWorldRayDirectionNV");
+  add(BuiltInObjectRayOriginKHR, "BuiltInObjectRayOriginKHR");
   add(BuiltInObjectRayOriginNV, "BuiltInObjectRayOriginNV");
+  add(BuiltInObjectRayDirectionKHR, "BuiltInObjectRayDirectionKHR");
   add(BuiltInObjectRayDirectionNV, "BuiltInObjectRayDirectionNV");
+  add(BuiltInRayTminKHR, "BuiltInRayTminKHR");
   add(BuiltInRayTminNV, "BuiltInRayTminNV");
+  add(BuiltInRayTmaxKHR, "BuiltInRayTmaxKHR");
   add(BuiltInRayTmaxNV, "BuiltInRayTmaxNV");
+  add(BuiltInInstanceCustomIndexKHR, "BuiltInInstanceCustomIndexKHR");
   add(BuiltInInstanceCustomIndexNV, "BuiltInInstanceCustomIndexNV");
+  add(BuiltInObjectToWorldKHR, "BuiltInObjectToWorldKHR");
   add(BuiltInObjectToWorldNV, "BuiltInObjectToWorldNV");
+  add(BuiltInWorldToObjectKHR, "BuiltInWorldToObjectKHR");
   add(BuiltInWorldToObjectNV, "BuiltInWorldToObjectNV");
   add(BuiltInHitTNV, "BuiltInHitTNV");
+  add(BuiltInHitKindKHR, "BuiltInHitKindKHR");
   add(BuiltInHitKindNV, "BuiltInHitKindNV");
+  add(BuiltInIncomingRayFlagsKHR, "BuiltInIncomingRayFlagsKHR");
   add(BuiltInIncomingRayFlagsNV, "BuiltInIncomingRayFlagsNV");
+  add(BuiltInRayGeometryIndexKHR, "BuiltInRayGeometryIndexKHR");
   add(BuiltInWarpsPerSMNV, "BuiltInWarpsPerSMNV");
   add(BuiltInSMCountNV, "BuiltInSMCountNV");
   add(BuiltInWarpIDNV, "BuiltInWarpIDNV");
   add(BuiltInSMIDNV, "BuiltInSMIDNV");
   add(BuiltInMax, "BuiltInMax");
+  add(internal::BuiltInSubDeviceIDINTEL, "BuiltInSubDeviceIDINTEL");
+  add(internal::BuiltInGlobalHWThreadIDINTEL, "BuiltInGlobalHWThreadIDINTEL");
 }
 SPIRV_DEF_NAMEMAP(BuiltIn, SPIRVBuiltInNameMap)
 
@@ -353,8 +390,15 @@ template <> inline void SPIRVMap<Capability, std::string>::init() {
   add(CapabilityGroupNonUniformQuad, "GroupNonUniformQuad");
   add(CapabilityShaderLayer, "ShaderLayer");
   add(CapabilityShaderViewportIndex, "ShaderViewportIndex");
+  add(CapabilityFragmentShadingRateKHR, "FragmentShadingRateKHR");
   add(CapabilitySubgroupBallotKHR, "SubgroupBallotKHR");
   add(CapabilityDrawParameters, "DrawParameters");
+  add(CapabilityWorkgroupMemoryExplicitLayoutKHR,
+      "WorkgroupMemoryExplicitLayoutKHR");
+  add(CapabilityWorkgroupMemoryExplicitLayout8BitAccessKHR,
+      "WorkgroupMemoryExplicitLayout8BitAccessKHR");
+  add(CapabilityWorkgroupMemoryExplicitLayout16BitAccessKHR,
+      "WorkgroupMemoryExplicitLayout16BitAccessKHR");
   add(CapabilitySubgroupVoteKHR, "SubgroupVoteKHR");
   add(CapabilityStorageBuffer16BitAccess, "StorageBuffer16BitAccess");
   add(CapabilityStorageUniformBufferBlock16, "StorageUniformBufferBlock16");
@@ -378,11 +422,17 @@ template <> inline void SPIRVMap<Capability, std::string>::init() {
   add(CapabilitySignedZeroInfNanPreserve, "SignedZeroInfNanPreserve");
   add(CapabilityRoundingModeRTE, "RoundingModeRTE");
   add(CapabilityRoundingModeRTZ, "RoundingModeRTZ");
+  add(CapabilityRayQueryProvisionalKHR, "RayQueryProvisionalKHR");
+  add(CapabilityRayQueryKHR, "RayQueryKHR");
+  add(CapabilityRayTraversalPrimitiveCullingKHR,
+      "RayTraversalPrimitiveCullingKHR");
+  add(CapabilityRayTracingKHR, "RayTracingKHR");
   add(CapabilityFloat16ImageAMD, "Float16ImageAMD");
   add(CapabilityImageGatherBiasLodAMD, "ImageGatherBiasLodAMD");
   add(CapabilityFragmentMaskAMD, "FragmentMaskAMD");
   add(CapabilityStencilExportEXT, "StencilExportEXT");
   add(CapabilityImageReadWriteLodAMD, "ImageReadWriteLodAMD");
+  add(CapabilityInt64ImageEXT, "Int64ImageEXT");
   add(CapabilityShaderClockKHR, "ShaderClockKHR");
   add(CapabilitySampleMaskOverrideCoverageNV, "SampleMaskOverrideCoverageNV");
   add(CapabilityGeometryShaderPassthroughNV, "GeometryShaderPassthroughNV");
@@ -453,10 +503,9 @@ template <> inline void SPIRVMap<Capability, std::string>::init() {
       "PhysicalStorageBufferAddresses");
   add(CapabilityPhysicalStorageBufferAddressesEXT,
       "PhysicalStorageBufferAddressesEXT");
-  add(CapabilityAtomicFloat32AddEXT, "AtomicFloat32AddEXT");
-  add(CapabilityAtomicFloat64AddEXT, "AtomicFloat64AddEXT");
   add(CapabilityComputeDerivativeGroupLinearNV,
       "ComputeDerivativeGroupLinearNV");
+  add(CapabilityRayTracingProvisionalKHR, "RayTracingProvisionalKHR");
   add(CapabilityCooperativeMatrixNV, "CooperativeMatrixNV");
   add(CapabilityFragmentShaderSampleInterlockEXT,
       "FragmentShaderSampleInterlockEXT");
@@ -477,9 +526,12 @@ template <> inline void SPIRVMap<Capability, std::string>::init() {
   add(CapabilityFunctionPointersINTEL, "FunctionPointersINTEL");
   add(CapabilityIndirectReferencesINTEL, "IndirectReferencesINTEL");
   add(CapabilityAsmINTEL, "AsmINTEL");
+  add(CapabilityAtomicFloat32MinMaxEXT, "AtomicFloat32MinMaxEXT");
+  add(CapabilityAtomicFloat64MinMaxEXT, "AtomicFloat64MinMaxEXT");
+  add(CapabilityAtomicFloat16MinMaxEXT, "AtomicFloat16MinMaxEXT");
   add(CapabilityVectorComputeINTEL, "VectorComputeINTEL");
   add(CapabilityVectorAnyINTEL, "VectorAnyINTEL");
-  add(internal::CapabilityOptimizationHintsINTEL, "OptimizationHintsINTEL");
+  add(CapabilityExpectAssumeKHR, "ExpectAssumeKHR");
   add(CapabilitySubgroupAvcMotionEstimationINTEL,
       "SubgroupAvcMotionEstimationINTEL");
   add(CapabilitySubgroupAvcMotionEstimationIntraINTEL,
@@ -496,22 +548,48 @@ template <> inline void SPIRVMap<Capability, std::string>::init() {
       "ArbitraryPrecisionFloatingPointINTEL");
   add(CapabilityUnstructuredLoopControlsINTEL, "UnstructuredLoopControlsINTEL");
   add(CapabilityFPGALoopControlsINTEL, "FPGALoopControlsINTEL");
-  add(CapabilityBlockingPipesINTEL, "BlockingPipesINTEL");
-  add(CapabilityFPGARegINTEL, "FPGARegINTEL");
   add(CapabilityKernelAttributesINTEL, "KernelAttributesINTEL");
   add(CapabilityFPGAKernelAttributesINTEL, "FPGAKernelAttributesINTEL");
+  add(CapabilityFPGAMemoryAccessesINTEL, "FPGAMemoryAccessesINTEL");
+  add(CapabilityFPGAClusterAttributesINTEL, "FPGAClusterAttributesINTEL");
+  add(CapabilityLoopFuseINTEL, "LoopFuseINTEL");
   add(CapabilityFPGABufferLocationINTEL, "FPGABufferLocationINTEL");
   add(CapabilityArbitraryPrecisionFixedPointINTEL,
       "ArbitraryPrecisionFixedPointINTEL");
   add(CapabilityUSMStorageClassesINTEL, "USMStorageClassesINTEL");
-  add(CapabilityFPGAMemoryAccessesINTEL, "FPGAMemoryAccessesINTEL");
   add(CapabilityIOPipesINTEL, "IOPipeINTEL");
-  add(CapabilityFPGAClusterAttributesINTEL, "FPGAClusterAttributesINTEL");
-  add(CapabilityLoopFuseINTEL, "LoopFuseINTEL");
-  add(CapabilityMax, "Max");
+  add(CapabilityBlockingPipesINTEL, "BlockingPipesINTEL");
+  add(CapabilityFPGARegINTEL, "FPGARegINTEL");
+  add(CapabilityDotProductInputAllKHR, "DotProductInputAllKHR");
+  add(CapabilityDotProductInput4x8BitKHR, "DotProductInput4x8BitKHR");
+  add(CapabilityDotProductInput4x8BitPackedKHR,
+      "DotProductInput4x8BitPackedKHR");
+  add(CapabilityDotProductKHR, "DotProductKHR");
+  add(CapabilityBitInstructions, "BitInstructions");
+  add(CapabilityAtomicFloat32AddEXT, "AtomicFloat32AddEXT");
+  add(CapabilityAtomicFloat64AddEXT, "AtomicFloat64AddEXT");
   add(CapabilityLongConstantCompositeINTEL, "LongConstantCompositeINTEL");
-  add(CapabilityFastCompositeINTEL, "FastCompositeINTEL");
+  add(CapabilityAtomicFloat16AddEXT, "AtomicFloat16AddEXT");
+  add(CapabilityDebugInfoModuleINTEL, "DebugInfoModuleINTEL");
+
+  // From spirv_internal.hpp
+  add(internal::CapabilityFPGADSPControlINTEL, "FPGADSPControlINTEL");
+  add(internal::CapabilityFastCompositeINTEL, "FastCompositeINTEL");
   add(internal::CapabilityOptNoneINTEL, "OptNoneINTEL");
+  add(internal::CapabilityMemoryAccessAliasingINTEL,
+      "MemoryAccessAliasingINTEL");
+  add(internal::CapabilityFPGAInvocationPipeliningAttributesINTEL,
+      "FPGAInvocationPipeliningAttributesINTEL");
+  add(internal::CapabilityTokenTypeINTEL, "TokenTypeINTEL");
+  add(internal::CapabilityRuntimeAlignedAttributeINTEL,
+      "RuntimeAlignedAttributeINTEL");
+  add(CapabilityMax, "Max");
+  add(internal::CapabilityFPArithmeticFenceINTEL, "FPArithmeticFenceINTEL");
+  add(internal::CapabilityBfloat16ConversionINTEL, "Bfloat16ConversionINTEL");
+  add(internal::CapabilityJointMatrixINTEL, "JointMatrixINTEL");
+  add(internal::CapabilityHWThreadQueryINTEL, "HWThreadQueryINTEL");
+  add(internal::CapabilityGlobalVariableDecorationsINTEL,
+      "GlobalVariableDecorationsINTEL");
 }
 SPIRV_DEF_NAMEMAP(Capability, SPIRVCapabilityNameMap)
 
