@@ -295,6 +295,7 @@ _SPIRV_OP(TypePipeStorage, 322)
 _SPIRV_OP(ConstantPipeStorage, 323)
 _SPIRV_OP(CreatePipeFromPipeStorage, 324)
 _SPIRV_OP(ModuleProcessed, 330)
+_SPIRV_OP(DecorateId, 332)
 _SPIRV_OP(GroupNonUniformElect, 333)
 _SPIRV_OP(GroupNonUniformAll, 334)
 _SPIRV_OP(GroupNonUniformAny, 335)
@@ -327,6 +328,12 @@ _SPIRV_OP(GroupNonUniformBitwiseXor, 361)
 _SPIRV_OP(GroupNonUniformLogicalAnd, 362)
 _SPIRV_OP(GroupNonUniformLogicalOr, 363)
 _SPIRV_OP(GroupNonUniformLogicalXor, 364)
+_SPIRV_OP(SDotKHR, 4450)
+_SPIRV_OP(UDotKHR, 4451)
+_SPIRV_OP(SUDotKHR, 4452)
+_SPIRV_OP(SDotAccSatKHR, 4453)
+_SPIRV_OP(UDotAccSatKHR, 4454)
+_SPIRV_OP(SUDotAccSatKHR, 4455)
 _SPIRV_OP(SubgroupShuffleINTEL, 5571)
 _SPIRV_OP(SubgroupShuffleDownINTEL, 5572)
 _SPIRV_OP(SubgroupShuffleUpINTEL, 5573)
@@ -337,11 +344,15 @@ _SPIRV_OP(SubgroupImageBlockReadINTEL, 5577)
 _SPIRV_OP(SubgroupImageBlockWriteINTEL, 5578)
 _SPIRV_OP(SubgroupImageMediaBlockReadINTEL, 5580)
 _SPIRV_OP(SubgroupImageMediaBlockWriteINTEL, 5581)
-_SPIRV_OP(ConstFunctionPointerINTEL, 5600)
+_SPIRV_OP(ConstantFunctionPointerINTEL, 5600)
 _SPIRV_OP(FunctionPointerCallINTEL, 5601)
 _SPIRV_OP(AsmTargetINTEL, 5609)
 _SPIRV_OP(AsmINTEL, 5610)
 _SPIRV_OP(AsmCallINTEL, 5611)
+_SPIRV_OP(AtomicFMinEXT, 5614)
+_SPIRV_OP(AtomicFMaxEXT, 5615)
+_SPIRV_OP(AssumeTrueKHR, 5630)
+_SPIRV_OP(ExpectKHR, 5631)
 _SPIRV_OP(VmeImageINTEL, 5699)
 _SPIRV_OP(TypeVmeImageINTEL, 5700)
 _SPIRV_OP(TypeAvcImePayloadINTEL, 5701)
