@@ -43,11 +43,9 @@
 #include "SPIRVDebug.h"
 #include "SPIRVExtInst.h"
 #include "SPIRVModule.h"
-#include <algorithm>
 #include <cctype>
 #include <cstdint>
 #include <iostream>
-#include <iterator>
 #include <string>
 #include <vector>
 
@@ -78,6 +76,8 @@ public:
   void validate() const;
   void ignore(size_t N);
   void ignoreInstruction();
+  std::vector<SPIRVEntry *>
+  getContinuedInstructions(const spv::Op ContinuedOpCode);
 
   std::istream &IS;
   SPIRVModule &M;
