@@ -53,9 +53,10 @@ SPIRVType *SPIRVType::getArrayElementType() const {
 
 uint64_t SPIRVType::getArrayLength() const {
   assert(OpCode == OpTypeArray && "Not array type");
-  return static_cast<const SPIRVTypeArray *>(this)
-      ->getLength()
-      ->getZExtIntValue();
+  const SPIRVTypeArray *AsArray = static_cast<const SPIRVTypeArray *>(this);
+  assert(AsArray->getLength()->getOpCode() == OpConstant &&
+         "getArrayLength can only be called with constant array lengths");
+  return AsArray->getLength()->getZExtIntValue();
 }
 
 SPIRVWord SPIRVType::getBitWidth() const {
@@ -110,18 +111,22 @@ SPIRVWord SPIRVType::getVectorComponentCount() const {
 }
 
 SPIRVType *SPIRVType::getVectorComponentType() const {
-  assert(OpCode == OpTypeVector && "Not vector type");
-  return static_cast<const SPIRVTypeVector *>(this)->getComponentType();
+  if (OpCode == OpTypeVector)
+    return static_cast<const SPIRVTypeVector *>(this)->getComponentType();
+  if (OpCode == internal::OpTypeJointMatrixINTEL)
+    return static_cast<const SPIRVTypeJointMatrixINTEL *>(this)->getCompType();
+  assert(0 && "getVectorComponentType(): Not a vector or joint matrix type");
+  return nullptr;
 }
 
 SPIRVWord SPIRVType::getMatrixColumnCount() const {
   assert(OpCode == OpTypeMatrix && "Not matrix type");
-  return static_cast<const SPIRVTypeMatrix *const>(this)->getColumnCount();
+  return static_cast<const SPIRVTypeMatrix *>(this)->getColumnCount();
 }
 
 SPIRVType *SPIRVType::getMatrixColumnType() const {
   assert(OpCode == OpTypeMatrix && "Not matrix type");
-  return static_cast<const SPIRVTypeMatrix *const>(this)->getColumnType();
+  return static_cast<const SPIRVTypeMatrix *>(this)->getColumnType();
 }
 
 SPIRVType *SPIRVType::getScalarType() const {
@@ -150,7 +155,8 @@ bool SPIRVType::isTypeArray() const { return OpCode == OpTypeArray; }
 bool SPIRVType::isTypeBool() const { return OpCode == OpTypeBool; }
 
 bool SPIRVType::isTypeComposite() const {
-  return isTypeVector() || isTypeArray() || isTypeStruct();
+  return isTypeVector() || isTypeArray() || isTypeStruct() ||
+         isTypeJointMatrixINTEL();
 }
 
 bool SPIRVType::isTypeFloat(unsigned Bits) const {
@@ -192,6 +198,10 @@ bool SPIRVType::isTypeStruct() const { return OpCode == OpTypeStruct; }
 
 bool SPIRVType::isTypeVector() const { return OpCode == OpTypeVector; }
 
+bool SPIRVType::isTypeJointMatrixINTEL() const {
+  return OpCode == internal::OpTypeJointMatrixINTEL;
+}
+
 bool SPIRVType::isTypeVectorBool() const {
   return isTypeVector() && getVectorComponentType()->isTypeBool();
 }
@@ -246,8 +256,7 @@ SPIRVTypeArray::SPIRVTypeArray(SPIRVModule *M, SPIRVId TheId,
 void SPIRVTypeArray::validate() const {
   SPIRVEntry::validate();
   ElemType->validate();
-  assert(getValue(Length)->getType()->isTypeInt() &&
-         get<SPIRVConstant>(Length)->getZExtIntValue() > 0);
+  assert(getValue(Length)->getType()->isTypeInt());
 }
 
 SPIRVConstant *SPIRVTypeArray::getLength() const {
@@ -265,4 +274,18 @@ void SPIRVTypeForwardPointer::decode(std::istream &I) {
   SPIRVId PointerId;
   Decoder >> PointerId >> SC;
 }
+
+SPIRVTypeJointMatrixINTEL::SPIRVTypeJointMatrixINTEL(
+    SPIRVModule *M, SPIRVId TheId, SPIRVType *CompType, SPIRVValue *Rows,
+    SPIRVValue *Columns, SPIRVValue *Layout, SPIRVValue *Scope)
+    : SPIRVType(M, FixedWC, OC, TheId), CompType(CompType), Rows(Rows),
+      Columns(Columns), Layout(Layout), Scope(Scope) {}
+
+SPIRVTypeJointMatrixINTEL::SPIRVTypeJointMatrixINTEL()
+    : SPIRVType(OC), CompType(nullptr), Rows(nullptr), Columns(nullptr),
+      Layout(nullptr), Scope(nullptr) {}
+
+_SPIRV_IMP_ENCDEC6(SPIRVTypeJointMatrixINTEL, Id, CompType, Rows, Columns,
+                   Layout, Scope)
+
 } // namespace SPIRV
