@@ -95,6 +95,7 @@ public:
   bool isTypeSampler() const;
   bool isTypeStruct() const;
   bool isTypeVector() const;
+  bool isTypeJointMatrixINTEL() const;
   bool isTypeVectorInt() const;
   bool isTypeVectorFloat() const;
   bool isTypeVectorBool() const;
@@ -1057,5 +1058,35 @@ protected:
   _SPIRV_DEF_ENCDEC1(Id)
 };
 
+class SPIRVTypeJointMatrixINTEL : public SPIRVType {
+  SPIRVType *CompType;
+  SPIRVValue *Rows;
+  SPIRVValue *Columns;
+  SPIRVValue *Layout;
+  SPIRVValue *Scope;
+
+public:
+  const static Op OC = internal::OpTypeJointMatrixINTEL;
+  const static SPIRVWord FixedWC = 7;
+  // Complete constructor
+  SPIRVTypeJointMatrixINTEL(SPIRVModule *M, SPIRVId TheId, SPIRVType *CompType,
+                            SPIRVValue *Rows, SPIRVValue *Columns,
+                            SPIRVValue *Layout, SPIRVValue *Scope);
+  // Incomplete constructor
+  SPIRVTypeJointMatrixINTEL();
+  _SPIRV_DCL_ENCDEC
+  llvm::Optional<ExtensionID> getRequiredExtension() const override {
+    return ExtensionID::SPV_INTEL_joint_matrix;
+  }
+  SPIRVCapVec getRequiredCapability() const override {
+    return {internal::CapabilityJointMatrixINTEL};
+  }
+  SPIRVType *getCompType() const { return CompType; }
+  SPIRVValue *getLayout() const { return Layout; }
+  SPIRVValue *getRows() const { return Rows; }
+  SPIRVValue *getColumns() const { return Columns; }
+  SPIRVValue *getScope() const { return Scope; }
+};
+
 } // namespace SPIRV
 #endif // SPIRV_LIBSPIRV_SPIRVTYPE_H
