@@ -41,7 +41,12 @@ enum InternalOp {
   IOpTypeTokenINTEL = 6113,
   IOpConvertFToBF16INTEL = 6116,
   IOpConvertBF16ToFINTEL = 6117,
+  IOpTypeJointMatrixINTEL = 6119,
+  IOpJointMatrixLoadINTEL = 6120,
+  IOpJointMatrixStoreINTEL = 6121,
+  IOpJointMatrixMadINTEL = 6122,
   IOpArithmeticFenceINTEL = 6145,
+  IOpJointMatrixWorkItemLengthINTEL = 6410,
   IOpPrev = OpMax - 2,
   IOpForward
 };
@@ -54,7 +59,11 @@ enum InternalDecoration {
   IDecMaxConcurrencyINTEL = 5918,
   IDecPipelineEnableINTEL = 5919,
   IDecRuntimeAlignedINTEL = 5940,
-  IDecCallableFunctionINTEL = 6087
+  IDecCallableFunctionINTEL = 6087,
+  IDecHostAccessINTEL = 6147,
+  IDecInitModeINTEL = 6148,
+  IDecImplementInCSRINTEL = 6149,
+  IDecArgumentAttributeINTEL = 6409
 };
 
 enum InternalCapability {
@@ -66,7 +75,10 @@ enum InternalCapability {
   ICapOptNoneINTEL = 6094,
   ICapTokenTypeINTEL = 6112,
   ICapBfloat16ConversionINTEL = 6115,
-  ICapFPArithmeticFenceINTEL = 6144
+  ICapabilityJointMatrixINTEL = 6118,
+  ICapabilityHWThreadQueryINTEL = 6134,
+  ICapFPArithmeticFenceINTEL = 6144,
+  ICapGlobalVariableDecorationsINTEL = 6146
 };
 
 enum InternalFunctionControlMask { IFunctionControlOptNoneINTELMask = 0x10000 };
@@ -76,13 +88,35 @@ enum InternalMemoryAccessMask {
   IMemAccessNoAliasINTELMask = 0x20000
 };
 
-enum InternalExecutionMode { IExecModeFastCompositeKernelINTEL = 6088 };
+enum InternalExecutionMode {
+  IExecModeFastCompositeKernelINTEL = 6088,
+  IExecModeStreamingInterfaceINTEL = 6154
+};
 
 enum InternalLoopControlMask { ILoopControlLoopCountINTELMask = 0x1000000 };
 
 constexpr LinkageType LinkageTypeInternal =
     static_cast<LinkageType>(ILTInternal);
 
+enum InternalJointMatrixLayout { RowMajor, ColumnMajor, PackedA, PackedB };
+
+enum InternalBuiltIn {
+  IBuiltInSubDeviceIDINTEL = 6135,
+  IBuiltInGlobalHWThreadIDINTEL = 6136,
+};
+
+#define _SPIRV_OP(x, y) constexpr x x##y = static_cast<x>(I##x##y);
+_SPIRV_OP(Capability, JointMatrixINTEL)
+_SPIRV_OP(Op, TypeJointMatrixINTEL)
+_SPIRV_OP(Op, JointMatrixLoadINTEL)
+_SPIRV_OP(Op, JointMatrixStoreINTEL)
+_SPIRV_OP(Op, JointMatrixMadINTEL)
+_SPIRV_OP(Op, JointMatrixWorkItemLengthINTEL)
+_SPIRV_OP(Capability, HWThreadQueryINTEL)
+_SPIRV_OP(BuiltIn, SubDeviceIDINTEL)
+_SPIRV_OP(BuiltIn, GlobalHWThreadIDINTEL)
+#undef _SPIRV_OP
+
 constexpr Op OpForward = static_cast<Op>(IOpForward);
 constexpr Op OpAliasDomainDeclINTEL = static_cast<Op>(IOpAliasDomainDeclINTEL);
 constexpr Op OpAliasScopeDeclINTEL = static_cast<Op>(IOpAliasScopeDeclINTEL);
@@ -107,6 +141,14 @@ constexpr Decoration DecorationCallableFunctionINTEL =
     static_cast<Decoration>(IDecCallableFunctionINTEL);
 constexpr Decoration DecorationRuntimeAlignedINTEL =
     static_cast<Decoration>(IDecRuntimeAlignedINTEL);
+constexpr Decoration DecorationHostAccessINTEL =
+    static_cast<Decoration>(IDecHostAccessINTEL);
+constexpr Decoration DecorationInitModeINTEL =
+    static_cast<Decoration>(IDecInitModeINTEL);
+constexpr Decoration DecorationImplementInCSRINTEL =
+    static_cast<Decoration>(IDecImplementInCSRINTEL);
+constexpr Decoration DecorationArgumentAttributeINTEL =
+    static_cast<Decoration>(IDecArgumentAttributeINTEL);
 
 constexpr Capability CapabilityFastCompositeINTEL =
     static_cast<Capability>(ICapFastCompositeINTEL);
@@ -126,6 +168,8 @@ constexpr Capability CapabilityFPArithmeticFenceINTEL =
     static_cast<Capability>(ICapFPArithmeticFenceINTEL);
 constexpr Capability CapabilityBfloat16ConversionINTEL =
     static_cast<Capability>(ICapBfloat16ConversionINTEL);
+constexpr Capability CapabilityGlobalVariableDecorationsINTEL =
+    static_cast<Capability>(ICapGlobalVariableDecorationsINTEL);
 
 constexpr FunctionControlMask FunctionControlOptNoneINTELMask =
     static_cast<FunctionControlMask>(IFunctionControlOptNoneINTELMask);
@@ -140,6 +184,8 @@ constexpr MemoryAccessMask MemoryAccessNoAliasINTELMask =
 
 constexpr ExecutionMode ExecutionModeFastCompositeKernelINTEL =
     static_cast<ExecutionMode>(IExecModeFastCompositeKernelINTEL);
+constexpr ExecutionMode ExecutionModeStreamingInterfaceINTEL =
+    static_cast<ExecutionMode>(IExecModeStreamingInterfaceINTEL);
 
 constexpr LoopControlMask LoopControlLoopCountINTELMask =
     static_cast<LoopControlMask>(ILoopControlLoopCountINTELMask);
